# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Romain Gauthier <romain@itaapy.com>
#          Hervé Cauwelier <herve@itaapy.com>
#          David Versmisse <david.versmisse@itaapy.com>
#          Jerome Dumonteil <jerome.dumonteil@itaapy.com>

from collections.abc import Iterable

import pytest

from odfdo.cell import Cell
from odfdo.column import Column
from odfdo.document import Document
from odfdo.row import Row
from odfdo.table import Table
from odfdo.table_cache import RowCache, _erase_map_once, _insert_map_once


@pytest.fixture
def table(samples) -> Iterable[Table]:
    # simpletable :
    #   1	1	1	2	3	3	3
    #   1	1	1	2	3	3	3
    #   1	1	1	2	3	3	3
    #   1   2	3	4	5	6	7
    document = Document(samples("simple_table.ods"))
    yield document.body.get_table(name="Example1")


def test_empty_row_repeat(table):
    row = Row(repeated=5)
    table.insert_row(2, row)
    value = table.get_value((3, 3))
    assert value is None
    cell = table.get_cell((4, 5))
    assert cell.x == 4
    assert cell.y == 5
    values = table.get_row_values(1)
    assert values == [1, 1, 1, 2, 3, 3, 3]
    values = table.get_row_values(2)
    assert values == [None, None, None, None, None, None, None]
    values = table.get_row_values(6)
    assert values == [None, None, None, None, None, None, None]
    values = table.get_row_values(7)
    assert values == [1, 1, 1, 2, 3, 3, 3]
    assert table.height == 9


def test_row_repeat_twice(table):
    row = Row(repeated=6)
    table.insert_row(2, row)
    cell = Cell(value=333, repeated=2)
    assert cell.x is None
    assert cell.y is None
    row = Row()
    row.insert_cell(4, cell)
    assert row.get_values() == [None, None, None, None, 333, 333]
    assert row.width == 6
    row.repeated = 3
    table.set_row(4, row)
    # initial height  # *insert* row with repeated 5
    expected = 4 + 6 + 3 - 3
    # *set* row with repeated 3
    assert table.height == expected  # *set* row with repeated 3
    expected = [
        [1, 1, 1, 2, 3, 3, 3],
        [1, 1, 1, 2, 3, 3, 3],
        [None, None, None, None, None, None, None],
        [None, None, None, None, None, None, None],
        [None, None, None, None, 333, 333, None],
        [None, None, None, None, 333, 333, None],
        [None, None, None, None, 333, 333, None],
        [None, None, None, None, None, None, None],
        [1, 1, 1, 2, 3, 3, 3],
        [1, 2, 3, 4, 5, 6, 7],
    ]
    assert table.get_values() == expected
    row = table.get_row(6)
    assert row.get_values() == [None, None, None, None, 333, 333]
    assert row.width == 6
    cell = row.get_cell(5)
    assert cell.x == 5
    assert cell.y == 6
    assert cell.get_value() == 333


def test_cell_repeat(table):
    cell = Cell(value=55, repeated=5)
    table.insert_cell((2, 2), cell)
    expected = [
        [1, 1, 1, 2, 3, 3, 3, None, None, None, None, None],
        [1, 1, 1, 2, 3, 3, 3, None, None, None, None, None],
        [1, 1, 55, 55, 55, 55, 55, 1, 2, 3, 3, 3],
        [1, 2, 3, 4, 5, 6, 7, None, None, None, None, None],
    ]
    assert table.get_values() == expected
    assert table.width == 12


def test_clear_cache(table):
    table.clear()
    assert table.width == 0
    assert table.height == 0


def test_lonely_cell_add_cache(table):
    table.clear()
    table.set_value((6, 7), 1)
    assert table.width == 7
    assert table.height == 8
    cell = table.get_cell((6, 7))
    assert cell.x == 6
    assert cell.y == 7
    assert cell.get_value() == 1


def test_basic_spreadsheet_case():
    table = Table("Table", width=20, height=3)
    for _r in range(2):
        table.append_row()
    assert len(table.get_rows()) == 5
    vals = []
    for row in table.get_rows():
        vals.append(len(row.get_cells()))
    assert vals == [20, 20, 20, 0, 0]
    # last_row = table.get_row(-1)
    for r in range(3):
        for c in range(10):
            table.set_value((c, r), f"cell {c} {r}")
    for r in range(3, 5):
        for c in range(10):
            table.set_value((c, r), c * 100 + r)
    assert table.size == (20, 5)
    table.rstrip()
    assert table.size == (10, 5)
    assert len(table.get_row(-1).get_cells()) == 10


def test_basic_spreadsheet_case_property():
    table = Table("Table", width=20, height=3)
    for _r in range(2):
        table.append_row()
    assert len(table.get_rows()) == 5
    vals = []
    for row in table.get_rows():
        vals.append(len(row.cells))
    assert vals == [20, 20, 20, 0, 0]
    # last_row = table.get_row(-1)
    for r in range(3):
        for c in range(10):
            table.set_value((c, r), f"cell {c} {r}")
    for r in range(3, 5):
        for c in range(10):
            table.set_value((c, r), c * 100 + r)
    assert table.size == (20, 5)
    table.rstrip()
    assert table.size == (10, 5)
    assert len(table.get_row(-1).cells) == 10


def test_internal_bad_insert(table):
    cache = table._table_cache
    row_map = cache.row_map
    bad_val = len(cache.row_map) + 1
    with pytest.raises(IndexError):
        _insert_map_once(row_map, bad_val, 0)


def test_internal_bad_erase(table):
    cache = table._table_cache
    row_map = cache.row_map
    bad_val = len(cache.row_map)
    with pytest.raises(IndexError):
        _erase_map_once(row_map, bad_val)


def test_internal_del_row_0(table):
    table.delete_row(0)
    assert table.height == 3


def test_internal_row_cache_str(table):
    row = table.get_row(0)
    cache = row._row_cache
    assert str(cache) == "RC cell:[2, 3, 6]"


def test_internal_row_map_length(table):
    row = table.get_row(0)
    cache = row._row_cache
    assert len(cache.cell_map) == 3
    assert cache.cell_map_length() == 3


def test_internal_row_erase_cell(table):
    row = table.get_row(0)
    cache = row._row_cache
    row.delete_cell(0)
    assert cache.cell_map_length() == 3
    assert str(cache) == "RC cell:[1, 2, 5]"


def test_internal_row_erase_cell_2(table):
    row = table.get_row(0)
    cache = row._row_cache
    row.delete_cell(0)
    row.delete_cell(0)
    assert cache.cell_map_length() == 3
    assert str(cache) == "RC cell:[0, 1, 4]"


def test_internal_row_erase_cell_3(table):
    row = table.get_row(0)
    cache = row._row_cache
    row.delete_cell(0)
    row.delete_cell(0)
    row.delete_cell(0)
    assert cache.cell_map_length() == 2
    assert str(cache) == "RC cell:[0, 3]"


def test_internal_set_cell_in_cache(table):
    row = table.get_row(0)
    cache = row._row_cache
    cell = Cell()
    with pytest.raises(ValueError):
        cache.set_cell_in_cache(42, cell, row, True)


def test_internal_set_cell_in_cache2(table):
    row = table.get_row(0)
    cache = row._row_cache
    assert row.get_values() == [1, 1, 1, 2, 3, 3, 3]
    assert str(cache) == "RC cell:[2, 3, 6]"
    cell = row.get_cell(0)
    cell.value = 42
    row.set_cell(0, cell)
    assert str(cache) == "RC cell:[2, 3, 6]"
    assert row.get_values() == [42, 42, 42, 2, 3, 3, 3]


def test_internal_insert_cell_in_cache(table):
    row = table.get_row(0)
    cache = row._row_cache
    cell = Cell()
    with pytest.raises(ValueError):
        cache.insert_cell_in_cache(42, cell, row)


def test_internal_insert_cell_in_cache2(table):
    row = table.get_row(0)
    cache = row._row_cache
    cell = row.get_cell(0)
    cell.value = 42
    row.insert_cell(0, cell)
    assert str(cache) == "RC cell:[2, 5, 6, 9]"
    assert row.get_values() == [42, 42, 42, 1, 1, 1, 2, 3, 3, 3]


def test_internal_delete_cell_in_cache(table):
    row = table.get_row(0)
    cache = row._row_cache
    with pytest.raises(ValueError):
        cache.delete_cell_in_cache(42, row)


def test_internal_delete_cell_in_cache2():
    cache = RowCache()
    current_cached = cache.cached_cell(0)
    assert current_cached is None


def test_internal_cell_in_cache(table):
    row = table.get_row(0)
    cache = row._row_cache
    assert cache.cell_map == [2, 3, 6]
    assert len(cache.cell_elements) == 0
    row.get_cell(0)
    assert len(cache.cell_elements) == 1


def test_internal_cell_in_cache_then_delete(table):
    row = table.get_row(0)
    cache = row._row_cache
    assert cache.cell_map == [2, 3, 6]
    assert len(cache.cell_elements) == 0
    row.get_cell(0)
    row.delete_cell(0)
    assert len(cache.cell_elements) == 0


def test_internal_table_cache_str(table):
    cache = table._table_cache
    assert str(cache) == "TC row:[0, 1, 2, 3] col:[1, 3, 6]"


def test_internal_table_row_map_length(table):
    cache = table._table_cache
    assert len(cache.row_map) == 4


def test_internal_table_col_map_length(table):
    cache = table._table_cache
    assert len(cache.col_map) == 3
    assert cache.col_map_length() == 3


def test_internal_table_row_bad_idx(table):
    cache = table._table_cache
    assert cache.row_idx(1000) is None


def test_internal_table_col_bad_idx(table):
    cache = table._table_cache
    assert cache.col_idx(1000) is None


def test_internal_table_erase_row(table):
    cache = table._table_cache
    table.get_row(0)
    table.delete_row(0)
    assert len(cache.row_map) == 3
    assert str(cache) == "TC row:[0, 1, 2] col:[1, 3, 6]"


def test_internal_table_set_row_in_cache(table):
    cache = table._table_cache
    row = Row()
    with pytest.raises(ValueError):
        cache.set_row_in_cache(42, row, table, False)


def test_internal_table_set_col_in_cache(table):
    cache = table._table_cache
    col = Column()
    with pytest.raises(ValueError):
        cache.set_col_in_cache(42, col, table)


def test_internal_table_set_col_in_cache_ok(table):
    cache = table._table_cache
    col = table.get_column(0)
    table.insert_column(0, col)
    table.insert_column(-1, col)
    assert str(cache) == "TC row:[0, 1, 2, 3] col:[1, 3, 5, 7, 9, 10]"


def test_internal_table_set_col_in_cache_ok2(table):
    previous = table.width
    table.get_column(0)
    new_col = Column()
    table.set_column(0, new_col)
    assert table.width == previous


def test_internal_table_get_col_in_cache_0(table):
    cache = table._table_cache
    assert len(cache.col_elements) == 0


def test_internal_table_get_col_in_cache_1(table):
    cache = table._table_cache
    idx = cache.col_idx(0)
    assert idx == 0


def test_internal_table_get_col_in_cache_2(table):
    cache = table._table_cache
    col = cache.cached_col(0)
    assert col is None


def test_internal_table_get_col_in_cache_3(table):
    cache = table._table_cache
    col = table.get_column(0)
    col = cache.cached_col(0)
    assert isinstance(col, Column)


def test_internal_table_insert_row_in_cache(table):
    cache = table._table_cache
    row = Row()
    with pytest.raises(ValueError):
        cache.insert_row_in_cache(42, row, table)


def test_internal_table_insert_row_in_cache2(table):
    cache = table._table_cache
    row = table.get_row(0)
    table.insert_row(0, row)
    table.insert_row(0, row)
    assert str(cache) == "TC row:[0, 1, 2, 3, 4, 5] col:[1, 3, 6]"


def test_internal_table_insert_column_in_cache(table):
    cache = table._table_cache
    col = Column()
    with pytest.raises(ValueError):
        cache.insert_col_in_cache(42, col, table)


def test_internal_table_insert_column_in_cache2(table):
    cache = table._table_cache
    col = table.get_column(0)
    table.insert_column(0, col)
    table.insert_column(0, col)
    assert str(cache) == "TC row:[0, 1, 2, 3] col:[1, 3, 5, 7, 10]"


def test_internal_table_delete_row_in_cache(table):
    cache = table._table_cache
    with pytest.raises(ValueError):
        cache.delete_row_in_cache(42, table)


def test_internal_table_delete_col_in_cache(table):
    cache = table._table_cache
    with pytest.raises(ValueError):
        cache.delete_col_in_cache(42, table)


def test_internal_table_delete_col_in_cache_ok(table):
    cache = table._table_cache
    table.get_column(0)
    cache.delete_col_in_cache(0, table)



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Hervé Cauwelier <herve@itaapy.com>
#          Luis Belmar-Letelier <luis@itaapy.com>
#          David Versmisse <david.versmisse@itaapy.com>
#          Jerome Dumonteil <jerome.dumonteil@itaapy.com>


import pytest

from odfdo.const import ODF_EXTENSIONS
from odfdo.document import _container_from_template
from odfdo.utils import to_bytes


def test_bad_template():
    with pytest.raises(FileNotFoundError):
        _container_from_template("notexisting")


def test_text_template(tmp_path):
    container = _container_from_template("text")
    mimetype = container.get_part("mimetype")
    umimetype = container.mimetype
    assert mimetype == to_bytes(ODF_EXTENSIONS["odt"])
    assert umimetype == ODF_EXTENSIONS["odt"]


def test_spreadsheet_template(tmp_path):
    container = _container_from_template("spreadsheet")
    mimetype = container.get_part("mimetype")
    umimetype = container.mimetype
    assert mimetype == to_bytes(ODF_EXTENSIONS["ods"])
    assert umimetype == ODF_EXTENSIONS["ods"]


def test_presentation_template(tmp_path):
    container = _container_from_template("presentation")
    mimetype = container.get_part("mimetype")
    umimetype = container.mimetype
    assert mimetype == to_bytes(ODF_EXTENSIONS["odp"])
    assert umimetype == ODF_EXTENSIONS["odp"]


def test_drawing_template(tmp_path):
    container = _container_from_template("drawing")
    mimetype = container.get_part("mimetype")
    umimetype = container.mimetype
    assert mimetype == to_bytes(ODF_EXTENSIONS["odg"])
    assert umimetype == ODF_EXTENSIONS["odg"]


def test_bad_type():
    with pytest.raises(FileNotFoundError):
        _container_from_template("foobar")


def test_bad_type2():
    with pytest.raises(TypeError):
        _container_from_template(object)


def test_bad_type3():
    with pytest.raises(TypeError):
        _container_from_template(None)


def test_text_type():
    container = _container_from_template("text")
    mimetype = container.get_part("mimetype")
    umimetype = container.mimetype
    assert mimetype == to_bytes(ODF_EXTENSIONS["odt"])
    assert umimetype == ODF_EXTENSIONS["odt"]


def test_spreadsheet_type():
    container = _container_from_template("spreadsheet")
    mimetype = container.get_part("mimetype")
    umimetype = container.mimetype
    assert mimetype == to_bytes(ODF_EXTENSIONS["ods"])
    assert umimetype == ODF_EXTENSIONS["ods"]


def test_presentation_type():
    container = _container_from_template("presentation")
    mimetype = container.get_part("mimetype")
    umimetype = container.mimetype
    assert mimetype == to_bytes(ODF_EXTENSIONS["odp"])
    assert umimetype == ODF_EXTENSIONS["odp"]


def test_drawing_type():
    container = _container_from_template("drawing")
    mimetype = container.get_part("mimetype")
    umimetype = container.mimetype
    assert mimetype == to_bytes(ODF_EXTENSIONS["odg"])
    assert umimetype == ODF_EXTENSIONS["odg"]


def test_clone():
    container = _container_from_template("text")
    clone = container.clone
    assert clone.path is None



#!/usr/bin/env python
"""Insert an image (e.g. the logo of an event, organization or a Creative Commons
attribution) with size `x,y` at position `x2,y2` on a number of slides in a
presentation slide deck.
"""

import os
import sys
from pathlib import Path

# reading image size requires a graphic library
from PIL import Image

from odfdo import Document, Frame

_DOC_SEQUENCE = 250
OUTPUT_DIR = Path(__file__).parent / "recipes_output" / "add_logo"
TARGET = "presentation.odp"
DATA = Path(__file__).parent / "data"
SOURCE = "presentation_wo_logo.odp"
LOGO = DATA / "newlogo.png"


def read_source_document() -> Document:
    """Return the source Document."""
    try:
        source = sys.argv[1]
    except IndexError:
        source = DATA / SOURCE
    return Document(source)


def save_new(document: Document, name: str) -> None:
    """Save a recipe result Document."""
    OUTPUT_DIR.mkdir(parents=True, exist_ok=True)
    new_path = OUTPUT_DIR / name
    print("Saving:", new_path)
    document.save(new_path, pretty=True)


def make_image_size(path: Path, size: float) -> tuple[str, str]:
    """Returns the display size (width, height) from the image path and the
    largest dimension."""
    width, height = Image.open(path).size
    ratio = max(width / size, height / size)
    return (f"{width / ratio:.2f}cm", f"{height / ratio:.2f}cm")


def add_logo(presentation: Document) -> None:
    """Add an image on a presentation."""
    image_position = ("1.50cm", "1.50cm")
    svg_title = "New Logo"
    svg_description = "The new logo with blue background"

    image_size = make_image_size(LOGO, 4.0)
    presentation_body = presentation.body
    uri = presentation.add_file(LOGO)

    for slide in presentation_body.get_draw_pages():
        # Create a frame for the image
        image_frame = Frame.image_frame(
            image=uri,
            text="",  # Text over the image object
            size=image_size,  # Display size of image
            anchor_type="page",
            page_number=None,
            position=image_position,
            style=None,
        )
        image_frame.svg_title = svg_title
        image_frame.svg_description = svg_description
        slide.append(image_frame)


def main() -> None:
    document = read_source_document()
    add_logo(document)
    test_unit(document)
    save_new(document, TARGET)


def test_unit(document: Document) -> None:
    # only for test suite:
    if "ODFDO_TESTING" not in os.environ:
        return

    slides = document.body.get_draw_pages()
    assert len(slides) == 11
    for slide in slides:
        assert len(slide.get_images()) == 1


if __name__ == "__main__":
    main()



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python

from collections.abc import Iterable

import pytest

from odfdo.document import Document
from odfdo.paragraph import PageBreak, Paragraph
from odfdo.style import Style


@pytest.fixture
def document(samples) -> Iterable[Document]:
    yield Document(samples("pagebreak.odt"))


def test_add_style_page_break():
    document = Document("text")
    document.add_page_break_style()
    result = document.get_style("paragraph", "odfdopagebreak")
    assert isinstance(result, Style)
    assert result.get_properties()["fo:break-after"] == "page"


def test_add_style_page_break_twice():
    document = Document("text")
    document.add_page_break_style()
    document.add_page_break_style()
    result = document.get_style("paragraph", "odfdopagebreak")
    assert isinstance(result, Style)
    assert result.get_properties()["fo:break-after"] == "page"


def test_break_page_is_paragraph():
    page_break = PageBreak()
    assert isinstance(page_break, Paragraph)


def test_break_str():
    page_break = PageBreak()
    assert str(page_break) == "\n"


def test_break_page_as_style():
    page_break = PageBreak()
    assert page_break.style == "odfdopagebreak"


def test_style_stay_in_doc(document):
    result = document.get_style("paragraph", "odfdopagebreak")
    assert isinstance(result, Style)
    assert result.get_properties()["fo:break-after"] == "page"



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Hervé Cauwelier <herve@itaapy.com>
#          David Versmisse <david.versmisse@itaapy.com>

from csv import reader
from mimetypes import guess_type

from PIL import Image

from odfdo import (
    FIRST_CHILD,
    Cell,
    Column,
    Document,
    Frame,
    Header,
    Paragraph,
    Row,
    Table,
    __version__,
)


def use_case1(destination_file, smp_dir):
    print("odfdo test use case 1")
    print(f"version: {__version__}")
    print(f"Generating test file {destination_file}")

    document = Document("text")
    body = document.body

    for numero, path in enumerate(smp_dir.iterdir()):
        # print(numero, path)
        filename = path.name
        heading = Header(1, text=filename)
        body.append(heading)
        mimetype, _ = guess_type(filename)
        if not mimetype:
            mimetype = "application/octet-stream"
        if mimetype.startswith("image/"):
            # Add the image
            print(path)
            image_uri = document.add_file(path)
            print("->", image_uri)
            # compute size
            image = Image.open(path)
            width, height = image.size
            draw_size = (f"{width / 400:.2f}in", f"{height / 400:.2f}in")
            image_frame = Frame.image_frame(
                image_uri,
                size=draw_size,
                anchor_type="char",
            )
            paragraph = Paragraph("")
            paragraph.append(image_frame)
            body.append(paragraph)
            body.append(Paragraph(""))
        elif mimetype in ("text/csv", "text/comma-separated-values"):
            print(mimetype)
            table = Table(f"table {numero}", style="Standard")
            size = 0
            with open(path) as file:
                csv = reader(file)
                for line in csv:
                    size = max(size, len(line))
                    row = Row()
                    for value in line:
                        cell = Cell(value)
                        row.append_cell(cell)
                    table.append_row(row)
                for _i in range(size):
                    column = Column(style="Standard")
                    table.insert(column, FIRST_CHILD)
            body.append(table)
        else:
            paragraph = Paragraph("Not image or csv", style="Standard")
            body.append(paragraph)

    destination_file.parent.mkdir(parents=True, exist_ok=True)
    document.save(destination_file, pretty=True)


def test_use_case1(tmp_path, samples_dir):
    path = tmp_path / "use_case_1.odt"
    use_case1(path, samples_dir)
    assert path.is_file()



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python

from collections.abc import Iterable

import pytest

from odfdo.paragraph import Paragraph


@pytest.fixture
def paragraph() -> Iterable[Paragraph]:
    txt = (
        "A test,\n   \twith a word and another word\n\n some \xe9 "
        "and \t and  5 spaces, and another word."
    )
    para = Paragraph()
    para.append(txt)
    yield para


def _test_search_word(paragraph):
    expected = (
        '<text:p>A test,<text:line-break/> <text:s text:c="2"/>'
        "<text:tab/>with a word and another word<text:line-break/>"
        "<text:line-break/> some é and <text:tab/> and <text:s/>"
        "5 spaces, and another word.</text:p>"
    )
    print(paragraph.serialize())
    assert paragraph.serialize() == expected


def test_search_test(paragraph):
    result = paragraph.search("test")
    print(result)
    assert result == 2


def test_search_none(paragraph):
    result = paragraph.search("bad")
    print(result)
    assert result is None


def test_search_word_1(paragraph):
    result = paragraph.search("word")
    assert result == 19


def test_search_search_first_test(paragraph):
    result = paragraph.search_first("test")
    print(result)
    assert result == (2, 6)


def test_search_first_none(paragraph):
    result = paragraph.search_first("bad")
    print(result)
    assert result is None


def test_search_first_word(paragraph):
    result = paragraph.search_first("word")
    print(result)
    assert result == (19, 23)


def test_text_at_test(paragraph):
    result1 = paragraph.search_first("test")
    result = paragraph.text_at(result1[0], result1[1])
    assert result == "test"


def test_text_at_none(paragraph):
    result = paragraph.text_at(-1, -1)
    assert result == ""


def test_text_at_all_1(paragraph):
    result = paragraph.text_at(-1)
    assert result == paragraph.inner_text


def test_text_at_all_2(paragraph):
    result = paragraph.text_at(0)
    assert result == paragraph.inner_text


def test_text_at_all_3(paragraph):
    result = paragraph.text_at(1)
    assert result == paragraph.inner_text[1:]


def test_text_at_word(paragraph):
    result1 = paragraph.search_first("word")
    result = paragraph.text_at(result1[0], result1[1])
    assert result == "word"


def test_search_test_regex(paragraph):
    result = paragraph.search(r"t..t")
    print(result)
    assert result == 2


def test_search_no_number_regex(paragraph):
    result = paragraph.search(r"[0-4]+")
    print(result)
    assert result is None


def test_search_word_1_regex(paragraph):
    result = paragraph.search(r"w[word]+")
    print(result)
    assert result == 19


def test_search_search_all_test(paragraph):
    result = paragraph.search_all("test")
    print(result)
    assert result == [(2, 6)]


def test_search_search_all_word(paragraph):
    result = paragraph.search_all("word")
    print(result)
    assert result == [(19, 23), (36, 40), (83, 87)]


def test_search_search_all_word_text_at(paragraph):
    result1 = paragraph.search_all("word")
    result = [paragraph.text_at(p[0], p[1]) for p in result1]
    assert result == ["word", "word", "word"]


def test_search_search_all_word_text_at_no_end(paragraph):
    result1 = paragraph.search_all("word")
    result = [paragraph.text_at(p[0], p[1]) for p in result1]
    assert result == ["word", "word", "word"]


def test_search_search_all_regex(paragraph):
    result = paragraph.search_all(r"a\w+")
    print(result)
    assert result == [
        (24, 27),
        (28, 35),
        (50, 53),
        (56, 59),
        (65, 69),
        (71, 74),
        (75, 82),
    ]


def test_search_search_all_regex_text_at(paragraph):
    result1 = paragraph.search_all(r"a\w+")
    result = [paragraph.text_at(p[0], p[1]) for p in result1]
    print(result)
    assert result == ["and", "another", "and", "and", "aces", "and", "another"]



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Romain Gauthier <romain@itaapy.com>
#          Hervé Cauwelier <herve@itaapy.com>
#          David Versmisse <david.versmisse@itaapy.com>
#          Jerome Dumonteil <jerome.dumonteil@itaapy.com>

from collections.abc import Iterable

import pytest

from odfdo.document import Document
from odfdo.element import Element
from odfdo.named_range import NamedRange
from odfdo.table import Table


@pytest.fixture
def table(samples) -> Iterable[Table]:
    # simpletable :
    #   1	1	1	2	3	3	3
    #   1	1	1	2	3	3	3
    #   1	1	1	2	3	3	3
    #   1   2	3	4	5	6	7
    document = Document(samples("simple_table.ods"))
    yield document.body.get_table(name="Example1")


@pytest.fixture
def table2(samples) -> Iterable[Table]:
    document = Document(samples("simple_table_named_range.ods"))
    yield document.body.get_table(name="Example1")


def test_create_bad_nr():
    with pytest.raises(ValueError):
        NamedRange()


def test_create_bad_nr_2():
    with pytest.raises(ValueError):
        NamedRange(" ", "A1", "tname")


def test_create_bad_nr_3():
    with pytest.raises(ValueError):
        NamedRange("A1", "A1", "tname")


def test_create_bad_nr_4():
    with pytest.raises(ValueError):
        NamedRange("a space", "A1", "tname")


def test_create_bad_nr_5():
    with pytest.raises(ValueError):
        NamedRange("===", "A1", "tname")


def test_create_bad_nr_6():
    with pytest.raises(ValueError):
        NamedRange("ok", "A1", "/ ")


def test_create_bad_nr_7():
    with pytest.raises(ValueError):
        NamedRange("ok", "A1", " ")


def test_create_bad_nr_8():
    with pytest.raises(ValueError):
        NamedRange("ok", "A1", "\\")


def test_create_bad_nr_9():
    with pytest.raises(ValueError):
        NamedRange("ok", "A1", "tname\nsecond line")


def test_create_bad_nr_10():
    with pytest.raises(TypeError):
        NamedRange("ok", "A1", 42)


def test_create_nr():
    nr = NamedRange("nr_name_ù", "A1:C2", "table name é", usage="filter")
    result = (
        """<table:named-range table:name="nr_name_ù" """
        """table:base-cell-address="$'table name é'.$A$1" """
        """table:cell-range-address="$'table name é'.$A$1:.$C$2" """
        """table:range-usable-as="filter"/>"""
    )
    assert nr.serialize() == result


def test_create_nr_minimal():
    minimal = "<table:named-range/>"
    nr = Element.from_tag(minimal)
    assert isinstance(nr, NamedRange)


def test_create_nr_minimal_2():
    minimal = (
        '<table:named-range table:name="nr_name_ù" table:range-usable-as="filter"/>'
    )
    nr = Element.from_tag(minimal)
    assert isinstance(nr, NamedRange)


def test_create_bad_nr_coordinate():
    with pytest.raises(ValueError):
        NamedRange("nr_name_ù", table_name="table name é")


def test_usage_1():
    nr = NamedRange("a123a", "A1:C2", "tablename")
    assert nr.usage is None
    nr.set_usage("blob")
    assert nr.usage is None


def test_usage_2():
    nr = NamedRange("nr_name", "A1:C2", "tablename")
    nr.set_usage("filter")
    assert nr.usage == "filter"
    nr.set_usage("blob")
    assert nr.usage is None


def test_usage_3():
    nr = NamedRange("nr_name", "A1:C2", "tablename")
    nr.set_usage("Print-Range")
    assert nr.usage == "print-range"
    nr.set_usage(None)
    assert nr.usage is None


def test_usage_4():
    nr = NamedRange("nr_name", "A1:C2", "tablename")
    nr.set_usage("repeat-column")
    assert nr.usage == "repeat-column"


def test_usage_5():
    nr = NamedRange("nr_name", "A1:C2", "tablename")
    nr.set_usage("repeat-row")
    assert nr.usage == "repeat-row"


def test_name_1():
    nr = NamedRange("nr_name", "A1:C2", "tablename")
    assert nr.name == "nr_name"


def test_name_2():
    NAME = "New_Name_ô"
    nr = NamedRange("nr_name", "A1:C2", "tablename")
    nr.name = f"  {NAME} "
    assert nr.name == NAME


def test_name_3():
    nr = NamedRange("nr_name", "A1:C2", "tablename")
    try:
        nr.name = "   "
        ok = True
    except Exception:
        ok = False
    assert ok is False


def test_table_name_1():
    nr = NamedRange("nr_name", "A1:C2", "tablename")
    assert nr.table_name == "tablename"


def test_table_name_2():
    nr = NamedRange("nr_name", "A1:C2", "tablename")
    nr.set_table_name("  new name ")
    assert nr.table_name == "new name"


def test_table_name_3():
    nr = NamedRange("nr_name", "A1:C2", "tablename")
    with pytest.raises(ValueError):
        nr.set_table_name("   ")


def test_range_1():
    nr = NamedRange("nr_name", "A1:C2", "tablename")
    with pytest.raises(ValueError):
        nr.set_range("   ")


def test_range_2():
    nr = NamedRange("nr_name", "A1:C2", "tablename")
    assert nr.crange == (0, 0, 2, 1)
    assert nr.start == (0, 0)
    assert nr.end == (2, 1)


def test_range_3():
    nr = NamedRange("nr_name", "A1", "tablename")
    assert nr.crange == (0, 0, 0, 0)
    assert nr.start == (0, 0)
    assert nr.end == (0, 0)


def test_range_4():
    nr = NamedRange("nr_name", (1, 2, 3, 4), "tablename")
    assert nr.crange == (1, 2, 3, 4)
    assert nr.start == (1, 2)
    assert nr.end == (3, 4)


def test_range_5():
    nr = NamedRange("nr_name", (5, 6), "tablename")
    assert nr.crange == (5, 6, 5, 6)
    assert nr.start == (5, 6)
    assert nr.end == (5, 6)


def test_range_6():
    nr = NamedRange("nr_name", "A1:C2", "tablename")
    nr.set_range("B3")
    assert nr.crange == (1, 2, 1, 2)
    assert nr.start == (1, 2)
    assert nr.end == (1, 2)


def test_range_7():
    nr = NamedRange("nr_name", "A1:C2", "tablename")
    nr.set_range("B3:b10")
    assert nr.crange == (1, 2, 1, 9)
    assert nr.start == (1, 2)
    assert nr.end == (1, 9)


def test_range_8():
    nr = NamedRange("nr_name", "A1:C2", "tablename")
    nr.set_range((1, 5, 0, 9))
    assert nr.crange == (1, 5, 0, 9)
    assert nr.start == (1, 5)
    assert nr.end == (0, 9)


def test_range_9():
    nr = NamedRange("nr_name", "A1:C2", "tablename")
    nr.set_range((0, 9))
    assert nr.crange == (0, 9, 0, 9)
    assert nr.start == (0, 9)
    assert nr.end == (0, 9)


def test_value_bad_1():
    nr = NamedRange("nr_name", "A1:C2", "tablename")
    with pytest.raises(ValueError):
        nr.get_values()


def test_value_bad_2():
    nr = NamedRange("nr_name", "A1:C2", "tablename")
    with pytest.raises(ValueError):
        nr.get_value()


def test_value_bad_3():
    nr = NamedRange("nr_name", "A1:C2", "tablename")
    with pytest.raises(ValueError):
        nr.set_values([[1, 2]])


def test_value_bad_4():
    nr = NamedRange("nr_name", "A1:C2", "tablename")
    with pytest.raises(ValueError):
        nr.set_value(42)


def test_body_table_get_1(table):
    assert table.get_named_ranges() == []


def test_body_table_get_2(table2):
    result = [nr.name for nr in table2.get_named_ranges()]
    assert result == ["nr_1", "nr_6"]


def test_body_table_get_3(table2):
    assert len(table2.get_named_ranges()) == 2


def test_body_table_get_4(table2):
    back_nr = table2.get_named_range("nr_1")
    assert back_nr.name == "nr_1"


def test_body_table_get_4_1(table2):
    back_nr = table2.get_named_range("nr_1xxx")
    assert back_nr is None


def test_body_table_get_4_2(table2):
    back_nr = table2.get_named_range("nr_6")
    assert back_nr.name == "nr_6"
    assert back_nr.table_name == "Example1"
    assert back_nr.start == (3, 2)
    assert back_nr.end == (5, 3)
    assert back_nr.crange == (3, 2, 5, 3)
    assert back_nr.usage == "print-range"


def test_body_table_get_5(table):
    back_nr = table.get_named_range("nr_1")
    assert back_nr is None


def test_body_table_rename_1(table2):
    back_nr = table2.get_named_range("nr_1")
    back_nr.name = "new_name"
    assert back_nr.name == "new_name"


def test_body_table_rename_2(table2):
    back_nr = table2.get_named_range("nr_1")
    back_nr.name = "new_name"
    back_nr2 = table2.get_named_range("nr_1")
    assert back_nr2 is None


def test_body_table_rename_3(table2):
    back_nr = table2.get_named_range("nr_1")
    back_nr.name = "new_name"
    back_nr2 = table2.get_named_range("new_name")
    assert isinstance(back_nr2, NamedRange)


def test_body_table_rename_4(table2):
    back_nr = table2.get_named_range("nr_1")
    back_nr.name = "new_name"
    back_nr2 = table2.get_named_range("new_name")
    expected = (
        '<table:named-range table:name="new_name" '
        'table:base-cell-address="$Example1.$C$1" '
        'table:cell-range-address="$Example1.$C$1"/>'
    )
    assert back_nr2.serialize() == expected


def test_body_table_rename_same(table2):
    table2.set_named_range(
        name="nr_1", table_name="Example1", crange="C5", usage="print-range"
    )
    nr = table2.get_named_range("nr_1")
    assert nr.name == "nr_1"
    assert nr.table_name == "Example1"
    assert nr.start == (2, 4)
    assert nr.end == (2, 4)
    assert nr.crange == (2, 4, 2, 4)
    assert nr.usage == "print-range"


def test_body_table_rename_same_2(table2):
    nr = table2.get_named_range("nr_6")
    nr.name = "nr_1"
    assert nr.name == "nr_1"
    assert nr.table_name == "Example1"
    assert nr.start == (3, 2)
    assert nr.end == (5, 3)
    assert nr.crange == (3, 2, 5, 3)
    assert nr.usage == "print-range"


def test_body_table_rename_same_3(table2):
    table2.set_named_range(
        name="nr_1", table_name="Example1", crange="C5", usage="print-range"
    )
    nr = table2.get_named_range("nr_1")
    nr.name = "nr_42"
    assert nr.name == "nr_42"
    assert nr.table_name == "Example1"


def test_body_table_set_0(table2):
    with pytest.raises(ValueError):
        table2.set_named_range("   ", "A1:C2")


def test_body_table_set_1(table2):
    table2.set_named_range("new", "A1:B1")
    result = [nr.name for nr in table2.get_named_ranges()]
    assert result == ["nr_1", "nr_6", "new"]


def test_body_table_set_3(table2):
    table2.set_named_range("new", "A1:B1")
    back_nr = table2.get_named_range("new")
    assert back_nr.usage is None
    assert back_nr.crange == (0, 0, 1, 0)
    assert back_nr.start == (0, 0)
    assert back_nr.end == (1, 0)
    assert back_nr.table_name == "Example1"
    # reset
    table2.set_named_range("new", "A1:c2")
    result = [nr.name for nr in table2.get_named_ranges()]
    assert result == ["nr_1", "nr_6", "new"]
    back_nr = table2.get_named_range("new")
    assert back_nr.usage is None
    assert back_nr.crange == (0, 0, 2, 1)
    assert back_nr.start == (0, 0)
    assert back_nr.end == (2, 1)
    assert back_nr.table_name == "Example1"


def test_body_table_name_set(table2):
    nr = table2.get_named_range("nr_6")
    nr.table_name = "badname"
    assert nr.table_name == "badname"


def test_body_table_name_set_2(table2):
    nr = table2.get_named_range("nr_6")
    nr.table_name = "badname"
    with pytest.raises(ValueError):
        nr.get_values()


def test_body_table_name_set_3(table2):
    nr = table2.get_named_range("nr_6")
    nr.table_name = "badname"
    with pytest.raises(ValueError):
        nr.get_value()


def test_body_table_name_set_4(table2):
    nr = table2.get_named_range("nr_6")
    nr.table_name = "badname"
    with pytest.raises(ValueError):
        nr.set_values([1, 2, 3, 4])


def test_body_table_name_set_5(table2):
    nr = table2.get_named_range("nr_6")
    nr.table_name = "badname"
    with pytest.raises(ValueError):
        nr.set_value(1)


def test_body_table_delete_1(table2):
    table2.delete_named_range("xxx")
    result = [nr.name for nr in table2.get_named_ranges()]
    assert result == ["nr_1", "nr_6"]


def test_body_table_delete_2(table2):
    table2.delete_named_range("nr_1")
    result = [nr.name for nr in table2.get_named_ranges()]
    assert result == ["nr_6"]


def test_body_table_delete_3(table2):
    table2.set_named_range("new", "A1:c2")
    result = [nr.name for nr in table2.get_named_ranges()]
    assert result == ["nr_1", "nr_6", "new"]
    table2.delete_named_range("nr_1")
    table2.delete_named_range("nr_6")
    result = [nr.name for nr in table2.get_named_ranges()]
    assert result == ["new"]
    table2.delete_named_range("new")
    result = [nr.name for nr in table2.get_named_ranges()]
    assert result == []
    table2.delete_named_range("new")
    table2.delete_named_range("xxx")
    table2.set_named_range("hop", "A1:C2")
    result = [nr.name for nr in table2.get_named_ranges()]
    assert result == ["hop"]
    table2.set_named_range("hop", "A2:d8")
    result = [nr.name for nr in table2.get_named_ranges()]
    assert result == ["hop"]
    nr = table2.get_named_range("hop")
    assert nr.crange == (0, 1, 3, 7)


def test_body_table_delete_none():
    table = Table("table")
    doc = Document("ods")
    doc.body.clear()
    doc.body.append(table)
    assert table.delete_named_range("nothing") is None


def test_body_table_get_value_1(table2):
    result = table2.get_named_range("nr_1").get_value()
    assert result == 1


def test_body_table_get_value_2(table2):
    result = table2.get_named_range("nr_1").get_value(get_type=True)
    assert result == (1, "float")


def test_body_table_get_value_3(table2):
    result = table2.get_named_range("nr_1").get_values()
    assert result == [[1]]


def test_body_table_get_value_4(table2):
    result = table2.get_named_range("nr_1").get_values(flat=True)
    assert result == [1]


def test_body_table_get_value_5(table2):
    result = table2.get_named_range("nr_6").get_values(flat=True)
    assert result == [2, 3, 3, 4, 5, 6]


def test_body_table_get_value_6(table2):
    result = table2.get_named_range("nr_6").get_value()
    assert result == 2


def test_body_table_set_value_1(table2):
    table2.get_named_range("nr_6").set_value("AAA")
    assert table2.get_value("D3") == "AAA"
    assert table2.get_value("E3") == 3


def test_body_table_set_value_2(table2):
    table2.get_named_range("nr_6").set_values([[10, 11, 12], [13, 14, 15]])
    assert table2.get_values() == [
        [1, 1, 1, 2, 3, 3, 3],
        [1, 1, 1, 2, 3, 3, 3],
        [1, 1, 1, 10, 11, 12, 3],
        [1, 2, 3, 13, 14, 15, 7],
    ]


def test_body_change_name_table(table2):
    table2.name = "new table"
    result = [nr.name for nr in table2.get_named_ranges()]
    assert result == ["nr_1", "nr_6"]
    back_nr = table2.get_named_range("nr_6")
    assert back_nr.name == "nr_6"
    assert back_nr.table_name == "new table"
    assert back_nr.start == (3, 2)
    assert back_nr.end == (5, 3)
    assert back_nr.crange == (3, 2, 5, 3)
    assert back_nr.usage == "print-range"


def test_body_set_named_range_bad_name(samples):
    document = Document(samples("simple_table_named_range.ods"))
    with pytest.raises(ValueError):
        document.body.set_named_range("", "", "")


def test_body_set_named_range_bad_table_name(samples):
    document = Document(samples("simple_table_named_range.ods"))
    with pytest.raises(ValueError):
        document.body.set_named_range("name", "", "")



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Romain Gauthier <romain@itaapy.com>
#          Hervé Cauwelier <herve@itaapy.com>

from collections import namedtuple
from collections.abc import Iterable
from datetime import datetime

import pytest

from odfdo.document import Document
from odfdo.tracked_changes import TrackedChanges

Sample = namedtuple("Sample", ["doc", "changes"])


@pytest.fixture
def sample(samples) -> Iterable[Sample]:
    document = Document(samples("tracked_changes.odt"))
    yield Sample(doc=document, changes=document.body.get_tracked_changes())


def test_instance_tracked_changes(sample):
    assert isinstance(sample.changes, TrackedChanges)


def test_tracked_changes_property(sample, samples):
    document2 = Document(samples("tracked_changes.odt"))
    sample2 = Sample(doc=document2, changes=document2.body.tracked_changes)
    assert isinstance(sample2.changes, TrackedChanges)
    assert sample.changes.serialize() == sample2.changes.serialize()


def test_get_changed_region_list(sample):
    regions = sample.changes.get_changed_regions()
    assert len(regions) == 3


def test_get_changed_region_list_creator(sample):
    creator = "Romain Gauthier"
    tracked_changes = sample.changes
    regions = tracked_changes.get_changed_regions(creator=creator)
    expected = (
        '<text:changed-region xml:id="ct140266191788864" text:id="ct140266191788864">\n'
        "  <text:deletion>\n"
        "    <office:change-info>\n"
        "      <dc:creator>Romain Gauthier</dc:creator>\n"
        "      <dc:date>2009-08-21T19:27:00</dc:date>\n"
        "    </office:change-info>\n"
        '    <text:p text:style-name="Standard">les</text:p>\n'
        "  </text:deletion>\n"
        "</text:changed-region>\n"
    )
    assert regions[0].serialize(pretty=True) == expected


def test_get_changed_region_list_date(sample):
    date = datetime(2009, 8, 21, 17, 27, 00)
    tracked_changes = sample.changes
    regions = tracked_changes.get_changed_regions(date=date)
    expected = (
        '<text:changed-region xml:id="ct140266193096800" text:id="ct140266193096800">\n'
        "  <text:deletion>\n"
        "    <office:change-info>\n"
        "      <dc:creator>David Versmisse</dc:creator>\n"
        "      <dc:date>2009-08-21T17:27:00</dc:date>\n"
        "    </office:change-info>\n"
        '    <text:p text:style-name="Standard">amis,</text:p>\n'
        "  </text:deletion>\n"
        "</text:changed-region>\n"
    )
    assert regions[0].serialize(pretty=True) == expected


def test_get_changed_region_list_regex(sample):
    tracked_changes = sample.changes
    regions = tracked_changes.get_changed_regions(content="amis")
    expected = (
        '<text:changed-region xml:id="ct140266193096800" text:id="ct140266193096800">\n'
        "  <text:deletion>\n"
        "    <office:change-info>\n"
        "      <dc:creator>David Versmisse</dc:creator>\n"
        "      <dc:date>2009-08-21T17:27:00</dc:date>\n"
        "    </office:change-info>\n"
        '    <text:p text:style-name="Standard">amis,</text:p>\n'
        "  </text:deletion>\n"
        "</text:changed-region>\n"
    )
    assert regions[0].serialize(pretty=True) == expected


def test_get_changed_region_by_id(sample):
    tracked_changes = sample.changes
    region = tracked_changes.get_changed_region(text_id="ct140266193096576")
    expected = (
        '<text:changed-region xml:id="ct140266193096576" text:id="ct140266193096576">\n'
        "  <text:insertion>\n"
        "    <office:change-info>\n"
        "      <dc:creator>Hervé Cauwelier</dc:creator>\n"
        "      <dc:date>2009-08-21T18:27:00</dc:date>\n"
        "    </office:change-info>\n"
        "  </text:insertion>\n"
        "</text:changed-region>\n"
    )
    assert region.serialize(pretty=True) == expected


def test_get_changed_region_by_creator(sample):
    creator = "David Versmisse"
    tracked_changes = sample.changes
    region = tracked_changes.get_changed_region(creator=creator)
    expected = (
        '<text:changed-region xml:id="ct140266193096800" text:id="ct140266193096800">\n'
        "  <text:deletion>\n"
        "    <office:change-info>\n"
        "      <dc:creator>David Versmisse</dc:creator>\n"
        "      <dc:date>2009-08-21T17:27:00</dc:date>\n"
        "    </office:change-info>\n"
        '    <text:p text:style-name="Standard">amis,</text:p>\n'
        "  </text:deletion>\n"
        "</text:changed-region>\n"
    )
    assert region.serialize(pretty=True) == expected


def test_get_changed_region_by_date(sample):
    date = datetime(2009, 8, 21, 18, 27, 00)
    tracked_changes = sample.changes
    region = tracked_changes.get_changed_region(date=date)
    expected = (
        '<text:changed-region xml:id="ct140266193096576" text:id="ct140266193096576">\n'
        "  <text:insertion>\n"
        "    <office:change-info>\n"
        "      <dc:creator>Hervé Cauwelier</dc:creator>\n"
        "      <dc:date>2009-08-21T18:27:00</dc:date>\n"
        "    </office:change-info>\n"
        "  </text:insertion>\n"
        "</text:changed-region>\n"
    )
    assert region.serialize(pretty=True) == expected


def test_get_changed_region_by_content(sample):
    tracked_changes = sample.changes
    region = tracked_changes.get_changed_region(content=r"les")
    expected = (
        '<text:changed-region xml:id="ct140266191788864" text:id="ct140266191788864">\n'
        "  <text:deletion>\n"
        "    <office:change-info>\n"
        "      <dc:creator>Romain Gauthier</dc:creator>\n"
        "      <dc:date>2009-08-21T19:27:00</dc:date>\n"
        "    </office:change-info>\n"
        '    <text:p text:style-name="Standard">les</text:p>\n'
        "  </text:deletion>\n"
        "</text:changed-region>\n"
    )
    assert region.serialize(pretty=True) == expected


def test_get_changes_ids(sample):
    paragraph = sample.doc.body.get_paragraph(content=r"Bonjour")
    changes_ids = paragraph.get_changes_ids()
    expected = ["ct140266191788864", "ct140266193096576", "ct140266193096800"]
    assert changes_ids == expected


def test_repr(sample):
    assert repr(sample.changes) == "<TrackedChanges tag=text:tracked-changes>"


def test_str(sample):
    result = str(sample.changes)
    assert "Romain" in result
    assert "2009-08-21" in result



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2013 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: David Versmisse <david.versmisse@itaapy.com>
#          Hervé Cauwelier <herve@itaapy.com>
#          Romain Gauthier <romain@itaapy.com>
#          Jerome Dumonteil <jerome.dumonteil@itaapy.com>
"""Tab class for "text:tab" tag."""

from __future__ import annotations

from typing import Any

from .element import Element, PropDef, PropDefBool, register_element_class
from .mixin_md import MDTab


class Tab(MDTab, Element):
    """Representation of a tabulation, "text:tab".

    This element represents the [UNICODE] tab character (HORIZONTAL TABULATION,
    U+0009). The position attribute indicates the number of the tab-stop to
    which a tab character refers, where position 0 marks the start margin of
    a paragraph. Layout-oriented consumers should determine tab positions based
    on style information.
    """

    _tag = "text:tab"
    _properties: tuple[PropDef | PropDefBool, ...] = (
        PropDef("position", "text:tab-ref"),
    )

    def __init__(self, position: int | None = None, **kwargs: Any) -> None:
        """Create a tabulation element "text:tab".

        Args:
            position: The position of the tab-stop. If provided,
                must be a non-negative integer.
        """
        super().__init__(**kwargs)
        if self._do_init and position is not None and position >= 0:
            self.position = str(position)

    def __str__(self) -> str:
        return "\t"

    @property
    def text(self) -> str:
        """Get the text content, which is always a tab character.

        Returns:
            str: The tab character ("\t").
        """
        return "\t"

    @text.setter
    def text(self, text: str | None) -> None:
        """Setting text for a tab is a no-op as it always represents a tab character."""
        pass


Tab._define_attribut_property()

register_element_class(Tab)



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Hervé Cauwelier <herve@itaapy.com>

from collections.abc import Iterable

import pytest

from odfdo import Element
from odfdo.bookmark import Bookmark, BookmarkEnd, BookmarkStart
from odfdo.document import Document
from odfdo.paragraph import Paragraph

ZOE = "你好 Zoé"


@pytest.fixture
def sample_body(samples) -> Iterable[Element]:
    document = Document(samples("bookmark.odt"))
    yield document.body


def test_create_bookmark():
    bookmark = Bookmark(ZOE)
    expected = f'<text:bookmark text:name="{ZOE}"/>'
    assert bookmark.serialize() == expected


def test_bookmark_str():
    bookmark = Bookmark(ZOE)
    assert str(bookmark) == ""


def test_create_bookmark_start():
    bookmark_start = BookmarkStart(ZOE)
    expected = f'<text:bookmark-start text:name="{ZOE}"/>'
    assert bookmark_start.serialize() == expected


def test_create_bookmark_end():
    bookmark_end = BookmarkEnd(ZOE)
    expected = f'<text:bookmark-end text:name="{ZOE}"/>'
    assert bookmark_end.serialize() == expected


def test_get_bookmark_main(sample_body):
    para = sample_body.get_paragraph()
    bookmark = Bookmark(ZOE)
    para.append(bookmark)
    read_book = sample_body.get_bookmark(name=ZOE)
    expected = f'<text:bookmark text:name="{ZOE}"/>'
    assert read_book.serialize() == expected


def test_get_bookmark_list(sample_body):
    result = sample_body.get_bookmarks()
    assert len(result) == 1
    element = result[0]
    expected = '<text:bookmark text:name="Repère de texte"/>'
    assert element.serialize() == expected


def test_get_bookmark_start(sample_body):
    para = sample_body.get_paragraph()
    bookmark_start = BookmarkStart(ZOE)
    para.append(bookmark_start)
    get = sample_body.get_bookmark_start(name=ZOE)
    expected = f'<text:bookmark-start text:name="{ZOE}"/>'
    assert get.serialize() == expected


def test_get_bookmark_start_list(sample_body):
    bookmark_start = BookmarkStart(ZOE)
    para = sample_body.get_paragraph()
    para.append(bookmark_start)
    get = sample_body.get_bookmark_starts()[0]
    expected = f'<text:bookmark-start text:name="{ZOE}"/>'
    assert get.serialize() == expected


def test_get_bookmark_end(sample_body):
    para = sample_body.get_paragraph()
    bookmark_end = BookmarkEnd(ZOE)
    para.append(bookmark_end)
    get = sample_body.get_bookmark_end(name=ZOE)
    expected = f'<text:bookmark-end text:name="{ZOE}"/>'
    assert get.serialize() == expected


def test_get_bookmark_end_list(sample_body):
    bookmark_end = BookmarkEnd(ZOE)
    para = sample_body.get_paragraph()
    para.append(bookmark_end)
    get = sample_body.get_bookmark_ends()[0]
    expected = f'<text:bookmark-end text:name="{ZOE}"/>'
    assert get.serialize() == expected


def test_set_bookmark_simple(sample_body):
    paragraph = sample_body.get_paragraph(position=-1)
    paragraph.set_bookmark("A bookmark")
    bookmark = paragraph.get_bookmark(name="A bookmark")
    assert bookmark is not None


def test_set_bookmark_with_after_without_position():
    paragraph = Paragraph("aa bb aa aa cc aa dd")
    paragraph.set_span(style="style", regex="bb aa aa")
    paragraph.set_span(style="style", regex="dd")
    paragraph.set_bookmark("bookmark", after="aa")
    expected = (
        '<text:p>aa<text:bookmark text:name="bookmark"/> '
        '<text:span text:style-name="style">bb aa aa'
        "</text:span>"
        ' cc aa <text:span text:style-name="style">dd</text:span>'
        "</text:p>"
    )
    assert paragraph.serialize() == expected


def test_set_bookmark_with_before():
    paragraph = Paragraph("aa bb aa aa cc aa dd")
    paragraph.set_span(style="style", regex="bb aa aa")
    paragraph.set_span(style="style", regex="dd")
    paragraph.set_bookmark("bookmark", before="aa", position=1)
    expected = (
        "<text:p>aa "
        '<text:span text:style-name="style">bb '
        '<text:bookmark text:name="bookmark"/>aa aa'
        "</text:span>"
        ' cc aa <text:span text:style-name="style">dd</text:span>'
        "</text:p>"
    )
    assert paragraph.serialize() == expected


def test_set_bookmark_with_after():
    paragraph = Paragraph("aa bb aa aa cc aa dd")
    paragraph.set_span(style="style", regex="bb aa aa")
    paragraph.set_span(style="style", regex="dd")
    paragraph.set_bookmark("bookmark", after="aa", position=1)
    expected = (
        "<text:p>aa "
        '<text:span text:style-name="style">bb '
        'aa<text:bookmark text:name="bookmark"/> aa'
        "</text:span>"
        ' cc aa <text:span text:style-name="style">dd</text:span>'
        "</text:p>"
    )
    assert paragraph.serialize() == expected


def test_set_bookmark_with_position():
    paragraph = Paragraph("aa bb aa aa cc aa dd")
    paragraph.set_span(style="style", regex="bb aa aa")
    paragraph.set_span(style="style", regex="dd")
    paragraph.set_bookmark("bookmark1", position=0)
    paragraph.set_bookmark("bookmark2", position=2)
    paragraph.set_bookmark("bookmark3", position=len("aa bb aa aa cc aa dd"))
    expected = (
        '<text:p><text:bookmark text:name="bookmark1"/>aa'
        '<text:bookmark text:name="bookmark2"/> '
        '<text:span text:style-name="style">bb aa aa</text:span>'
        ' cc aa <text:span text:style-name="style">dd'
        '<text:bookmark text:name="bookmark3"/></text:span>'
        "</text:p>"
    )
    assert paragraph.serialize() == expected


def test_set_bookmark_with_end():
    paragraph = Paragraph("aa bb aa aa cc aa dd")
    paragraph.set_span(style="style", regex="bb aa aa")
    paragraph.set_span(style="style", regex="dd")
    paragraph.set_bookmark("bookmark1", after="cc", position=-1)
    paragraph.set_bookmark("bookmark2", position=-1)
    expected = (
        "<text:p>aa "
        '<text:span text:style-name="style">'
        "bb aa aa"
        "</text:span>"
        ' cc<text:bookmark text:name="bookmark1"/> aa '
        '<text:span text:style-name="style">dd</text:span>'
        '<text:bookmark text:name="bookmark2"/>'
        "</text:p>"
    )
    assert paragraph.serialize() == expected


def test_set_bookmark_with_role():
    paragraph = Paragraph("aa")
    paragraph.set_bookmark("bookmark", role="start")
    paragraph.set_bookmark("bookmark", role="end", position=-1)
    expected = (
        "<text:p>"
        '<text:bookmark-start text:name="bookmark"/>'
        "aa"
        '<text:bookmark-end text:name="bookmark"/>'
        "</text:p>"
    )
    assert paragraph.serialize() == expected


def test_set_bookmark_with_content():
    paragraph = Paragraph("aa bb bb aa")
    paragraph.set_bookmark("bookmark", content="bb", position=1)
    expected = (
        "<text:p>aa bb "
        '<text:bookmark-start text:name="bookmark"/>'
        "bb"
        '<text:bookmark-end text:name="bookmark"/>'
        " aa"
        "</text:p>"
    )
    assert paragraph.serialize() == expected


def test_set_bookmark_with_content_str():
    paragraph = Paragraph("aa bb bb aa")
    paragraph.set_bookmark("bookmark", content="bb", position=1)
    assert str(paragraph) == "aa bb bb aa\n"


def test_set_bookmark_with_limits():
    paragraph = Paragraph("aa bb bb aa")
    paragraph.set_bookmark("bookmark", position=(6, 8))
    expected = (
        "<text:p>aa bb "
        '<text:bookmark-start text:name="bookmark"/>'
        "bb"
        '<text:bookmark-end text:name="bookmark"/>'
        " aa"
        "</text:p>"
    )
    assert paragraph.serialize() == expected


def test_repr(sample_body):
    para = sample_body.get_paragraph()
    bookmark = Bookmark(ZOE)
    para.append(bookmark)
    read_book = sample_body.get_bookmark(name=ZOE)
    assert repr(read_book) == "<Bookmark tag=text:bookmark>"


def test_str(sample_body):
    para = sample_body.get_paragraph()
    bookmark = Bookmark(ZOE)
    para.append(bookmark)
    read_book = sample_body.get_bookmark(name=ZOE)
    assert str(read_book) == ""



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Hervé Cauwelier <herve@itaapy.com>

from collections.abc import Iterable

import pytest

from odfdo import Element
from odfdo.document import Document
from odfdo.reference import (
    Reference,
    ReferenceMark,
    ReferenceMarkEnd,
    ReferenceMarkStart,
    remove_all_reference_marks,
    remove_reference_mark,
    strip_references,
)

ZOE = "你好 Zoé"


@pytest.fixture
def body1(samples) -> Iterable[Element]:
    document = Document(samples("bookmark.odt")).clone
    yield document.body


@pytest.fixture
def body2(samples) -> Iterable[Element]:
    document = Document(samples("base_text.odt")).clone
    yield document.body


def test_reference_class():
    ref = Reference()
    assert isinstance(ref, Reference)


def test_reference_args():
    ref = Reference(name="refname", ref_format="chapter")
    assert ref.name == "refname"
    assert ref.ref_format == "chapter"


def test_reference_args_default():
    ref = Reference(name="refname")
    assert ref.ref_format == "page"


def test_reference_args_default_none():
    # incomplete content:
    ref = Element.from_tag("<text:reference-ref/>")
    assert isinstance(ref, Reference)
    assert ref.ref_format is None


def test_reference_update(body1):
    marks = body1.get_reference_mark_starts()
    mark = marks[0]
    ref_name = mark.name
    ref = Reference(name=ref_name, ref_format="text")
    body1.append(ref)  # required
    mark_start = body1.get_reference_mark(name=ref_name)
    # same mark as before
    assert isinstance(mark_start, ReferenceMarkStart)
    ref_text = mark_start.referenced_text()
    expected = "Élise"
    assert ref_text == expected
    assert ref.text == ""
    ref.update()
    assert ref.text == expected


def test_reference_update_not(body1):
    marks = body1.get_reference_mark_starts()
    mark = marks[0]
    ref_name = mark.name
    ref = Reference(name=ref_name, ref_format="page")
    body1.append(ref)  # required
    mark_start = body1.get_reference_mark(name=ref_name)
    # same mark as before
    assert isinstance(mark_start, ReferenceMarkStart)
    ref_text = mark_start.referenced_text()
    expected = "Élise"
    assert ref_text == expected
    assert ref.text == ""
    ref.update()
    # only update for 'text' format
    assert ref.text == ""


def test_reference_update_not_bad(body1):
    ref_name = "bad name"
    ref = Reference(name=ref_name, ref_format="text")
    body1.append(ref)  # required
    mark_start = body1.get_reference_mark(name=ref_name)
    # same mark as before
    assert mark_start is None
    assert ref.text == ""
    ref.update()
    assert ref.text == ""


def test_create_reference_mark():
    reference_mark = ReferenceMark(ZOE)
    expected = f'<text:reference-mark text:name="{ZOE}"/>'
    assert reference_mark.serialize() == expected


def test_create_reference_mark_str():
    reference_mark = ReferenceMark(ZOE)
    assert str(reference_mark) == ""


def test_create_reference_mark_start():
    reference_mark_start = ReferenceMarkStart(ZOE)
    expected = f'<text:reference-mark-start text:name="{ZOE}"/>'
    assert reference_mark_start.serialize() == expected


def test_create_reference_mark_end():
    reference_mark_end = ReferenceMarkEnd(ZOE)
    expected = f'<text:reference-mark-end text:name="{ZOE}"/>'
    assert reference_mark_end.serialize() == expected


def test_get_reference_mark_single(body1):
    para = body1.get_paragraph()
    reference_mark = ReferenceMark(ZOE)
    para.append(reference_mark)
    get = body1.get_reference_mark_single(name=ZOE)
    expected = f'<text:reference-mark text:name="{ZOE}"/>'
    assert get.serialize() == expected


def test_get_reference_mark_single_list(body1):
    para = body1.get_paragraph()
    reference_mark = ReferenceMark(ZOE)
    para.append(reference_mark)
    get = body1.get_reference_marks_single()[0]
    expected = f'<text:reference-mark text:name="{ZOE}"/>'
    assert get.serialize() == expected


def test_get_reference_mark_start(body1):
    para = body1.get_paragraph()
    reference_mark_start = ReferenceMarkStart(ZOE)
    para.append(reference_mark_start)
    get = body1.get_reference_mark_start(name=ZOE)
    expected = f'<text:reference-mark-start text:name="{ZOE}"/>'
    assert get.serialize() == expected


def test_get_reference_mark_start_list(body1):
    result = body1.get_reference_mark_starts()
    assert len(result) == 1
    element = result[0]
    expected = '<text:reference-mark-start text:name="Nouvelle référence"/>'
    assert element.serialize() == expected


def test_get_reference_mark_end(body1):
    para = body1.get_paragraph()
    reference_mark_end = ReferenceMarkEnd(ZOE)
    para.append(reference_mark_end)
    get = body1.get_reference_mark_end(name=ZOE)
    expected = f'<text:reference-mark-end text:name="{ZOE}"/>'
    assert get.serialize() == expected


def test_get_reference_mark_end_referenced(body1):
    para = body1.get_paragraph()
    reference_mark_end = ReferenceMarkEnd(ZOE)
    para.append(reference_mark_end)
    ref_end = body1.get_reference_mark_end(name=ZOE)
    assert ref_end.referenced_text() == ""


def test_get_reference_mark_end_list(body1):
    result = body1.get_reference_mark_ends()
    assert len(result) == 1
    element = result[0]
    expected = '<text:reference-mark-end text:name="Nouvelle référence"/>'
    assert element.serialize() == expected


def test_get_referenced_1_odf(body2):
    para = body2.get_paragraph(content="of the second title")
    para.set_reference_mark("one", content="paragraph of the second title")
    ref = body2.get_reference_mark(name="one")
    referenced = ref.get_referenced()
    expected = (
        '<office:text><text:p text:style-name="Text_20_body">'
        "paragraph of the second title</text:p></office:text>"
    )
    assert referenced.serialize() == expected


def test_get_referenced_1_xml(body2):
    para = body2.get_paragraph(content="of the second title")
    para.set_reference_mark("one", content="paragraph of the second title")
    ref = body2.get_reference_mark(name="one")
    referenced = ref.get_referenced(as_xml=True)
    expected = (
        '<office:text><text:p text:style-name="Text_20_body">'
        "paragraph of the second title</text:p></office:text>"
    )
    assert referenced == expected


def test_get_referenced_1_list(body2):
    para = body2.get_paragraph(content="of the second title")
    para.set_reference_mark("one", content="paragraph of the second title")
    ref = body2.get_reference_mark(name="one")
    referenced = ref.get_referenced(as_list=True)
    assert isinstance(referenced, list)
    assert len(referenced) == 1
    expected = (
        '<text:p text:style-name="Text_20_body">paragraph of the second title</text:p>'
    )
    assert referenced[0].serialize() == expected


def test_get_referenced_multi_odf(body2):
    para = body2.get_paragraph(content="of the second title")
    para.set_reference_mark("one", content=para)
    para.set_reference_mark("two", position=(0, 7))
    ref = body2.get_reference_mark(name="two")
    referenced = ref.get_referenced()
    expected = (
        "<office:text>"
        '<text:p text:style-name="Text_20_body">'
        "This is"
        "</text:p></office:text>"
    )
    assert referenced.serialize() == expected


def test_get_referenced_multi_xml_dirty(body2):
    para = body2.get_paragraph(content="of the second title")
    para.set_reference_mark("one", content=para)
    para.set_reference_mark("two", position=(0, 7))
    ref = body2.get_reference_mark(name="one")
    referenced = ref.get_referenced(as_xml=True, clean=False)
    expected = (
        "<office:text>"
        '<text:p text:style-name="Text_20_body">'
        '<text:reference-mark-start text:name="two"/>'
        'This is<text:reference-mark-end text:name="two"/>'
        " the first paragraph of the second title."
        "</text:p>"
        "</office:text>"
    )
    assert referenced == expected


def test_get_referenced_multi_xml_clean(body2):
    para = body2.get_paragraph(content="of the second title")
    para.set_reference_mark("one", content=para)
    para.set_reference_mark("two", position=(0, 7))
    ref = body2.get_reference_mark(name="one")
    referenced = ref.get_referenced(as_xml=True, clean=True)
    expected = (
        "<office:text>"
        '<text:p text:style-name="Text_20_body">'
        "This is"
        " the first paragraph of the second title."
        "</text:p>"
        "</office:text>"
    )
    assert referenced == expected


def test_get_referenced_mix_xml(body2):
    para = body2.get_paragraph()
    para.set_reference_mark("one", content="first paragraph.")
    para.set_reference_mark("two", position=(0, 17))
    ref = body2.get_reference_mark(name="two")
    referenced = ref.get_referenced(as_xml=True)
    expected = (
        "<office:text>"
        '<text:p text:style-name="Text_20_body">'
        "This is the first"
        "</text:p>"
        "</office:text>"
    )
    assert referenced == expected


def test_get_referenced_header(body2):
    head = body2.get_header()
    head.set_reference_mark("one", content=head)
    ref = body2.get_reference_mark(name="one")
    referenced = ref.get_referenced(as_xml=True)
    expected = (
        "<office:text>"
        '<text:h text:style-name="Heading_20_1" '
        'text:outline-level="1">'
        '<text:span text:style-name="T1">'
        "odfdo"
        "</text:span>"
        " Test Case Document"
        "</text:h>"
        "</office:text>"
    )
    assert referenced == expected


def test_get_referenced_no_header(body2):
    head = body2.get_header()
    head.set_reference_mark("one", content=head)
    ref = body2.get_reference_mark(name="one")
    referenced = ref.get_referenced(as_xml=True, no_header=True)
    expected = (
        "<office:text><text:p>"
        '<text:span text:style-name="T1">'
        "odfdo"
        "</text:span> "
        "Test Case Document"
        "</text:p>"
        "</office:text>"
    )
    assert referenced == expected


def test_get_referenced_empty():
    ref = ReferenceMarkStart()
    with pytest.raises(ValueError):
        ref.get_referenced()


def test_reference_delete(body2):
    head = body2.get_header()
    head.set_reference_mark("one", content=head)
    ref = body2.get_reference_mark(name="one")
    ref.delete()
    ref2 = body2.get_reference_mark(name="one")
    assert ref2 is None


def test_strip_references(body1):
    marks = body1.get_reference_mark_starts()
    mark = marks[0]
    ref_name = mark.name
    ref = Reference(name=ref_name, ref_format="text")
    body1.append(ref)
    refs = body1.get_references()
    assert len(refs) == 1
    strip_references(body1)
    assert not body1.get_references()


def test_remove_all_reference_marks(body1):
    marks = body1.get_reference_mark_starts()
    assert len(marks) == 1
    remove_all_reference_marks(body1)
    assert not body1.get_reference_mark_starts()


def test_remove_reference_mark_1(body1):
    remove_reference_mark(body1, 0)
    assert not body1.get_reference_mark_starts()


def test_remove_reference_mark_2(body1):
    remove_reference_mark(body1, name="Nouvelle référence")
    assert not body1.get_reference_mark_starts()


def test_remove_reference_mark_3(body1):
    remove_reference_mark(body1, name="wrong")
    assert len(body1.get_reference_mark_starts()) == 1



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Romain Gauthier <romain@itaapy.com>
#          Hervé Cauwelier <herve@itaapy.com>
#          David Versmisse <david.versmisse@itaapy.com>
#          Jerome Dumonteil <jerome.dumonteil@itaapy.com>

from datetime import date, datetime, timedelta
from decimal import Decimal as dec

import pytest

from odfdo.cell import Cell


def test_bool():
    cell = Cell(True)
    expected = (
        '<table:table-cell office:value-type="boolean" '
        'calcext:value-type="boolean" '
        'office:boolean-value="true">'
        "<text:p>true</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_bool_repr():
    cell = Cell(True, text="VRAI")
    expected = (
        '<table:table-cell office:value-type="boolean" '
        'calcext:value-type="boolean" '
        'office:boolean-value="true">'
        "<text:p>VRAI</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_int():
    cell = Cell(23)
    expected = (
        '<table:table-cell office:value-type="float" '
        'calcext:value-type="float" '
        'office:value="23" calcext:value="23">'
        "<text:p>23</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_int_repr():
    cell = Cell(23, text="00023")
    expected = (
        '<table:table-cell office:value-type="float" '
        'calcext:value-type="float" '
        'office:value="23" calcext:value="23">'
        "<text:p>00023</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_float():
    cell = Cell(3.141592654)
    expected = (
        '<table:table-cell office:value-type="float" '
        'calcext:value-type="float" '
        'office:value="3.141592654" calcext:value="3.141592654">'
        "<text:p>3.141592654</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_float_repr():
    cell = Cell(3.141592654, text="3,14")
    expected = (
        '<table:table-cell office:value-type="float" '
        'calcext:value-type="float" '
        'office:value="3.141592654" calcext:value="3.141592654">'
        "<text:p>3,14</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_decimal():
    cell = Cell(dec("2.718281828"))
    expected = (
        '<table:table-cell office:value-type="float" '
        'calcext:value-type="float" '
        'office:value="2.718281828" calcext:value="2.718281828">'
        "<text:p>2.718281828</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_decimal_repr():
    cell = Cell(dec("2.718281828"), text="2,72")
    expected = (
        '<table:table-cell office:value-type="float" '
        'calcext:value-type="float" '
        'office:value="2.718281828" calcext:value="2.718281828">'
        "<text:p>2,72</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_date():
    cell = Cell(date(2009, 6, 30))
    expected = (
        '<table:table-cell office:value-type="date" '
        'calcext:value-type="date" '
        'office:date-value="2009-06-30">'
        "<text:p>2009-06-30</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_date_value():
    cell = Cell(date(2009, 6, 30))
    assert cell.value == datetime(2009, 6, 30)
    assert cell.date == date(2009, 6, 30)


def test_date_repr():
    cell = Cell(date(2009, 6, 30), text="30/6/2009")
    expected = (
        '<table:table-cell office:value-type="date" '
        'calcext:value-type="date" '
        'office:date-value="2009-06-30">'
        "<text:p>30/6/2009</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_datetime():
    cell = Cell(datetime(2009, 6, 30, 17, 33, 18))
    expected = (
        '<table:table-cell office:value-type="date" '
        'calcext:value-type="date" '
        'office:date-value="2009-06-30T17:33:18">'
        "<text:p>2009-06-30T17:33:18</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_datetime_value():
    cell = Cell(datetime(2009, 6, 30, 17, 33, 18))
    assert cell.value == datetime(2009, 6, 30, 17, 33, 18)


def test_datetime_repr():
    cell = Cell(datetime(2009, 6, 30, 17, 33, 18), text="30/6/2009 17:33")
    expected = (
        '<table:table-cell office:value-type="date" '
        'calcext:value-type="date" '
        'office:date-value="2009-06-30T17:33:18">'
        "<text:p>30/6/2009 17:33</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_str():
    cell = Cell("red")
    expected = (
        '<table:table-cell office:value-type="string" '
        'calcext:value-type="string" '
        'office:string-value="red">'
        "<text:p>red</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_str_repr():
    cell = Cell("red", text="Red")
    expected = (
        '<table:table-cell office:value-type="string" '
        'calcext:value-type="string" '
        'office:string-value="red">'
        "<text:p>Red</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_unicode():
    cell = Cell("Plato")
    expected = (
        '<table:table-cell office:value-type="string" '
        'calcext:value-type="string" '
        'office:string-value="Plato">'
        "<text:p>Plato</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_unicode_repr():
    cell = Cell("Plato", text="P.")
    expected = (
        '<table:table-cell office:value-type="string" '
        'calcext:value-type="string" '
        'office:string-value="Plato">'
        "<text:p>P.</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_timedelta():
    cell = Cell(timedelta(0, 8))
    expected = (
        '<table:table-cell office:value-type="time" '
        'calcext:value-type="time" '
        'office:time-value="PT00H00M08S">'
        "<text:p>PT00H00M08S</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_timedelta_value():
    cell = Cell(timedelta(0, 8))
    assert cell.value == timedelta(0, 8)


def test_timedelta_repr():
    cell = Cell(timedelta(0, 8), text="00:00:08")
    expected = (
        '<table:table-cell office:value-type="time" '
        'calcext:value-type="time" '
        'office:time-value="PT00H00M08S">'
        "<text:p>00:00:08</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_timedelta_style():
    cell = Cell("before", cell_type="time", style="bold")
    duration = timedelta(days=0, hours=2, minutes=15)
    cell.value = duration
    expected = (
        "<table:table-cell "
        'office:time-value="PT02H15M00S" '
        'office:value-type="time" '
        'table:style-name="bold" '
        'calcext:value-type="time">'
        "PT02H15M00S"
        "</table:table-cell>"
    )
    assert cell._canonicalize() == expected


def test_percentage():
    cell = Cell(90, cell_type="percentage")
    expected = (
        '<table:table-cell office:value-type="percentage" '
        'calcext:value-type="percentage" '
        'office:value="90" calcext:value="90">'
        "<text:p>9000 %</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_percentage_repr():
    cell = Cell(90, text="9000 %", cell_type="percentage")
    expected = (
        '<table:table-cell office:value-type="percentage" '
        'calcext:value-type="percentage" '
        'office:value="90" calcext:value="90">'
        "<text:p>9000 %</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_currency():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    expected = (
        '<table:table-cell office:value-type="currency" '
        'calcext:value-type="currency" '
        'office:value="1.54" office:currency="EUR">'
        "<text:p>1.54</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_currency_property():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    assert cell.currency == "EUR"
    cell.currency = "USD"
    assert cell.currency == "USD"
    cell.currency = None
    assert cell.currency is None


def test_currency_repr():
    cell = Cell(1.54, text="1,54 €", cell_type="currency", currency="EUR")
    expected = (
        '<table:table-cell office:value-type="currency" '
        'calcext:value-type="currency" '
        'office:value="1.54" office:currency="EUR">'
        "<text:p>1,54 €</text:p>"
        "</table:table-cell>"
    )
    assert cell.serialize() == expected


def test_style():
    cell = Cell(style="Monétaire")
    expected = '<table:table-cell table:style-name="Monétaire"/>'
    assert cell.serialize() == expected


def test_bad():
    with pytest.raises(TypeError):
        Cell([])


def test_string_value_property():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    assert cell.string == ""
    cell.clear()
    assert cell.string == ""
    cell.string = 25
    assert cell.string == "25"
    cell.string = "hop"
    assert cell.string == "hop"
    cell.string = None
    assert cell.string == ""
    assert cell.value == ""


def test_string_value_property2_false():
    cell = Cell()
    cell.clear()
    assert cell.string == ""
    cell.string = "hop"
    assert cell.string == "hop"
    cell.string = "false"
    assert cell.string == "false"
    cell.string = "False"
    assert cell.string == "False"
    cell.string = False
    assert cell.string == "False"


def test_string_value_property2_true():
    cell = Cell()
    cell.clear()
    assert cell.string == ""
    cell.string = "hop"
    assert cell.string == "hop"
    cell.string = "true"
    assert cell.string == "true"
    cell.string = "True"
    assert cell.string == "True"
    cell.string = True
    assert cell.string == "True"


def test_string_value_property2_none():
    cell = Cell()
    cell.clear()
    assert cell.string == ""
    cell.string = "hop"
    assert cell.string == "hop"
    cell.string = "none"
    assert cell.string == "none"
    cell.string = "None"
    assert cell.string == "None"
    cell.string = None
    assert cell.string == ""


def test_string_value_property2_number():
    cell = Cell()
    cell.clear()
    assert cell.string == ""
    cell.string = "1.23"
    assert cell.string == "1.23"
    cell.string = 1.23
    assert cell.string == "1.23"


def test_value_property_set_int():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    cell.value = 1
    assert cell.value == 1


def test_value_property_set_int_2():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    cell.value = "2"
    assert cell.value == "2"


def test_value_property_set_str():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    cell.value = "hop"
    assert cell.value == "hop"


def test_value_property_set_str_2():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    cell.value = "éû"
    assert cell.value == "éû"


def test_value_property_set_float():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    cell.value = 1.5
    assert cell.value == 1.5


def test_value_property_set_timedelta():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    cell.value = timedelta(50, 10)
    assert cell.value == timedelta(50, 10)


def test_value_property_set_timedelta_getter():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    cell.value = timedelta(50, 10)
    assert cell.duration == timedelta(50, 10)


def test_value_property_timedelta_getter_empty():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    assert cell.duration == timedelta(0)


def test_value_property_set_datetime():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    cell.value = datetime(2009, 6, 30, 0, 0)
    assert cell.value == datetime(2009, 6, 30, 0, 0)


def test_value_property_set_datetime_getter():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    cell.value = datetime(2009, 6, 30, 0, 0)
    assert cell.datetime == datetime(2009, 6, 30, 0, 0)


def test_value_property_datetime_getter_empty():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    assert cell.datetime == datetime.fromtimestamp(0)


def test_value_property_datetime_set():
    cell = Cell(datetime(2009, 6, 30, 0, 0), cell_type="date")
    cell.datetime = datetime(2009, 7, 30, 0, 0)
    assert cell.datetime == datetime(2009, 7, 30, 0, 0)


def test_value_property_date_set():
    cell = Cell(date(2009, 6, 30), cell_type="date")
    cell.date = date(2009, 7, 30)
    assert cell.datetime == datetime(2009, 7, 30)


def test_value_property_set_datetime_styled():
    cell = Cell("before", cell_type="string", style="bold")
    cell.value = datetime(2009, 6, 30, 0, 0)
    assert cell.datetime == datetime(2009, 6, 30, 0, 0)
    expected = (
        "<table:table-cell "
        'office:date-value="2009-06-30T00:00:00" '
        'office:value-type="date" '
        'table:style-name="bold">'
        "2009-06-30T00:00:00"
        "</table:table-cell>"
    )
    assert cell._canonicalize() == expected


def test_value_property_set_date():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    cell.value = date(2009, 6, 30)
    # return alwais a datetime as value
    assert cell.value == datetime(2009, 6, 30)
    assert cell.date == date(2009, 6, 30)


def test_value_property_set_date_getter():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    cell.value = date(2009, 6, 30)
    assert cell.date == date(2009, 6, 30)


def test_value_property_date_getter_empty():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    # return alwais a datetime as value
    assert cell.date == date.fromtimestamp(0)


def test_value_property_set_date_styled():
    cell = Cell("before", cell_type="string", style="bold")
    cell.value = date(2009, 6, 30)
    assert cell.date == date(2009, 6, 30)
    expected = (
        "<table:table-cell "
        'office:date-value="2009-06-30" '
        'office:value-type="date" '
        'table:style-name="bold">'
        "2009-06-30"
        "</table:table-cell>"
    )
    assert cell._canonicalize() == expected


def test_value_property_bytes():
    cell = Cell()
    cell.clear()
    assert cell.string == ""
    cell.value = "éû".encode()
    assert cell.value == "éû"


def test_value_property_none():
    cell = Cell()
    cell.clear()
    assert cell.string == ""
    cell.value = None
    assert cell.value is None
    cell.value = "hop"
    assert cell.value == "hop"
    cell.value = None
    assert cell.value is None


def test_value_property_false():
    cell = Cell()
    cell.clear()
    assert cell.string == ""
    cell.value = False
    assert cell.value is False
    cell.value = "False"
    assert cell.value == "False"
    cell.value = "false"
    assert cell.value == "false"


def test_value_property_true():
    cell = Cell()
    cell.clear()
    assert cell.string == ""
    cell.value = True
    assert cell.value is True
    cell.value = "True"
    assert cell.value == "True"
    cell.value = "true"
    assert cell.value == "true"


def test_value_property2():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    cell.value = "3"
    expected = (
        "<table:table-cell "
        'office:string-value="3" '
        'office:value-type="string">'
        "3"
        "</table:table-cell>"
    )
    assert cell._canonicalize() == expected


def test_string_property2():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    cell.string = "Le changement"
    expected = (
        "<table:table-cell "
        'office:string-value="Le changement" '
        'office:value-type="string">'
        "Le changement"
        "</table:table-cell>"
    )
    assert cell._canonicalize() == expected


def test_string_property_style():
    cell = Cell("content", cell_type="string", style="some_style")
    cell.string = "changed"
    expected = (
        "<table:table-cell "
        'office:string-value="changed" '
        'office:value-type="string" '
        'table:style-name="some_style" '
        'calcext:value-type="string">'
        "changed"
        "</table:table-cell>"
    )
    assert cell._canonicalize() == expected


def test_float_value_property():
    cell = Cell(1.50, cell_type="currency", currency="EUR")
    assert cell.float == 1.50
    assert cell.value == dec("1.50")
    cell.clear()
    assert cell.float == 0.0
    assert cell.value is None
    cell.string = 25
    assert cell.float == 25
    assert cell.value == "25"
    cell.string = "hop"
    assert cell.float == 0.0
    assert cell.value == "hop"
    cell.float = None
    assert cell.float == 0.0
    assert cell.value == dec("0.0")
    cell.float = 12
    assert cell.float == 12
    assert cell.value == dec(12)
    cell.float = -12.0
    assert cell.float == -12.0
    assert cell.value == dec("-12.0")


def test_decimal_value_property():
    cell = Cell(dec("1.50"), cell_type="currency", currency="EUR")
    assert cell.float == 1.50
    assert cell.value == dec("1.50")
    assert cell.float == 1.50
    assert cell.decimal == dec("1.50")


def test_decimal_value_property_2():
    cell = Cell(0.0, cell_type="currency", currency="EUR")
    cell.value = dec("1.56")
    assert cell.float == 1.56
    assert cell.value == dec("1.56")
    assert cell.decimal == dec("1.56")


def test_decimal_value_property_3():
    cell = Cell(0.0, cell_type="currency", currency="EUR")
    cell.decimal = "oops"
    assert cell.float == 0.0
    assert cell.value == dec("0.0")
    assert cell.decimal == dec("0.0")


def test_decimal_value_property_4():
    cell = Cell("before", cell_type="string", style="bold")
    cell.decimal = dec("3.14")
    assert cell.float == 3.14
    assert cell.value == dec("3.14")
    assert cell.decimal == dec("3.14")
    expected = (
        "<table:table-cell "
        'office:value="3.14" '
        'office:value-type="float" '
        'table:style-name="bold">'
        "3.14"
        "</table:table-cell>"
    )
    assert cell._canonicalize() == expected


def test_int_value_property():
    cell = Cell(0, cell_type="float")
    assert cell.float == 0.0
    assert cell.value == 0
    assert cell.int == 0


def test_int_value_property_2():
    cell = Cell(0, cell_type="float")
    cell.value = 4
    assert cell.float == 4.0
    assert cell.value == 4
    assert cell.int == 4


def test_int_value_property_3():
    cell = Cell(0, cell_type="float")
    cell.int = "oops"
    assert cell.float == 0.0
    assert cell.value == 0
    assert cell.int == 0


def test_int_property_4():
    cell = Cell("before", cell_type="string", style="bold")
    cell.int = 12
    expected = (
        "<table:table-cell "
        'office:value="12" '
        'office:value-type="float" '
        'table:style-name="bold">'
        "12"
        "</table:table-cell>"
    )
    assert cell._canonicalize() == expected


def test_float_property_2():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    cell.float = 12
    expected = (
        '<table:table-cell office:value="12.0" '
        'office:value-type="float">12.0'
        "</table:table-cell>"
    )
    assert cell._canonicalize() == expected


def test_float_property_3():
    cell = Cell("before", cell_type="string", style="bold")
    cell.float = 12.0
    expected = (
        "<table:table-cell "
        'office:value="12.0" '
        'office:value-type="float" '
        'table:style-name="bold">'
        "12.0"
        "</table:table-cell>"
    )
    assert cell._canonicalize() == expected


def test_bool_property():
    cell = Cell(0, cell_type="float")
    cell.bool = True
    assert cell.value is True
    assert cell.bool is True


def test_bool_property_as_numeric():
    cell = Cell(0, cell_type="float")
    cell.bool = True
    assert cell.int == 1
    assert cell.float == 1.0
    assert cell.decimal == dec("1")


def test_bool_property_2():
    cell = Cell(0, cell_type="float")
    cell.bool = False
    assert cell.value is False
    assert cell.bool is False


def test_bool_property_as_numeric_2():
    cell = Cell(0, cell_type="float")
    cell.bool = False
    assert cell.int == 0
    assert cell.float == 0.0
    assert cell.decimal == dec("0")


def test_bool_property_as_str_1():
    cell = Cell(0, cell_type="float")
    cell.bool = "True"
    assert cell.int == 1
    assert cell.float == 1.0
    assert cell.decimal == dec("1")


def test_bool_property_as_str_2():
    cell = Cell(0, cell_type="float")
    cell.bool = "true"
    assert cell.int == 1
    assert cell.float == 1.0
    assert cell.decimal == dec("1")


def test_bool_property_as_str_3():
    cell = Cell(0, cell_type="float")
    cell.bool = "False"
    assert cell.int == 0
    assert cell.float == 0.0
    assert cell.decimal == dec("0")


def test_bool_property_as_str_4():
    cell = Cell(0, cell_type="float")
    cell.bool = "false"
    assert cell.int == 0
    assert cell.float == 0.0
    assert cell.decimal == dec("0")


def test_bool_property_as_str_5():
    cell = Cell(0, cell_type="float")
    with pytest.raises(TypeError):
        cell.bool = "oops"


def test_bool_property_as_bytes_1():
    cell = Cell(0, cell_type="float")
    cell.bool = b"True"
    assert cell.int == 1
    assert cell.float == 1.0
    assert cell.decimal == dec("1")


def test_bool_property_as_bytes_2():
    cell = Cell(0, cell_type="float")
    cell.bool = b"true"
    assert cell.int == 1
    assert cell.float == 1.0
    assert cell.decimal == dec("1")


def test_bool_property_as_bytes_3():
    cell = Cell(0, cell_type="float")
    cell.bool = b"False"
    assert cell.int == 0
    assert cell.float == 0.0
    assert cell.decimal == dec("0")


def test_bool_property_as_bytes_4():
    cell = Cell(0, cell_type="float")
    cell.bool = b"false"
    assert cell.int == 0
    assert cell.float == 0.0
    assert cell.decimal == dec("0")


def test_bool_property_as_bytes_5():
    cell = Cell(0, cell_type="float")
    with pytest.raises(TypeError):
        cell.bool = b"oops"


def test_bool_property_as_object_1():
    cell = Cell(0, cell_type="float")
    cell.bool = []
    assert cell.bool is False
    assert cell.int == 0
    assert cell.float == 0.0
    assert cell.decimal == dec("0")


def test_bool_property_as_object_2():
    cell = Cell(0, cell_type="float")
    cell.bool = [1, 2, 3]
    assert cell.bool is True
    assert cell.int == 1
    assert cell.float == 1.0
    assert cell.decimal == dec("1")


def test_bool_property_as_bool():
    cell = Cell(0, cell_type="boolean")
    cell.bool = True
    assert cell.bool is True
    assert cell.int == 1
    assert cell.float == 1.0
    assert cell.decimal == dec("1")


def test_bool_property_style():
    cell = Cell("before", cell_type="string", style="bold")
    cell.value = True
    assert cell.bool is True

    expected = (
        "<table:table-cell "
        'office:boolean-value="true" '
        'office:value-type="boolean" '
        'table:style-name="bold">'
        "true"
        "</table:table-cell>"
    )
    assert cell._canonicalize() == expected


def test_decimal_property():
    cell = Cell(0, cell_type="float")
    cell.decimal = dec("1.43")
    assert cell.value == dec("1.43")
    assert cell.int == 1
    assert cell.float == 1.43
    assert cell.decimal == dec("1.43")


def test_float_property():
    cell = Cell(0, cell_type="float")
    cell.float = dec("1.43")
    assert cell.value == dec("1.43")
    assert cell.int == 1
    assert cell.float == 1.43
    assert cell.decimal == dec("1.43")


def test_int_property():
    cell = Cell(0, cell_type="float")
    cell.int = 5
    assert cell.value == 5
    assert cell.int == 5
    assert cell.float == 5.0
    assert cell.decimal == dec("5")


def test_int_property_bool_true():
    cell = Cell(0, cell_type="float")
    cell.int = 1
    assert cell.bool is True


def test_int_property_bool_false():
    cell = Cell(0, cell_type="float")
    cell.int = 0
    assert cell.bool is False


def test_bad_value():
    cell = Cell(1.54, cell_type="currency", currency="EUR")
    with pytest.raises(TypeError):
        cell.value = []



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2013 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
"""Command-line script to search and replace text in an ODF document.

This script finds all occurrences of a given regular expression pattern in an
ODF document and replaces them with a specified string.
"""

from __future__ import annotations

from argparse import ArgumentParser, Namespace

from odfdo import __version__
from odfdo.utils.script_utils import read_document, save_document

PROG = "odfdo-replace"


def configure_parser() -> ArgumentParser:
    description = (
        "Find and replace text in an ODF file using a regular expression pattern."
    )
    epilog = (
        "This tool supports standard regular expressions for pattern matching. "
        "Replacements can be made with or without preserving original formatting. "
        "Input can be from a specified file or standard input. "
        "Output can be to a specified file or standard output."
    )
    parser = ArgumentParser(prog=PROG, description=description, epilog=epilog)

    parser.add_argument(
        "--version",
        action="version",
        version=f"{PROG} v{__version__}",
    )
    parser.add_argument(
        "-i",
        "--input",
        action="store",
        dest="input_file",
        metavar="INPUT",
        required=False,
        help="input file. if option not present, read from stdin",
    )
    parser.add_argument(
        "-o",
        "--output",
        action="store",
        dest="output_file",
        metavar="OUTPUT",
        required=False,
        help="output file. if option not present, write to stdout",
    )
    parser.add_argument(
        "-f",
        "--formatted",
        action="store_true",
        default=False,
        help="keep replacement string format",
    )
    parser.add_argument(
        "pattern",
        action="store",
        help="search pattern (regular expression)",
    )
    parser.add_argument(
        "replacement",
        action="store",
        help="replacement text",
    )
    return parser


def parse_cli_args(cli_args: list[str] | None = None) -> Namespace:
    parser = configure_parser()
    return parser.parse_args(cli_args)


def main() -> None:
    args: Namespace = parse_cli_args()
    main_replace(args)


def main_replace(args: Namespace) -> None:
    try:
        search_replace(
            args.pattern,
            args.replacement,
            args.input_file,
            args.output_file,
            args.formatted,
        )
    except Exception as e:
        configure_parser().print_help()
        print()
        print(f"Error: {e.__class__.__name__}, {e}")
        raise SystemExit(1) from None


def search_replace(
    pattern: str,
    replacement: str,
    input_path: str | None,
    output_path: str | None,
    formatted: bool = False,
) -> None:
    document = read_document(input_path)
    body = document.body
    body.replace(pattern, replacement, formatted)
    save_document(document, output_path)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""Retrieve all the pictures embeded in an .odt file."""

import os
import sys
from pathlib import Path
from pprint import pformat

from odfdo import Document

_DOC_SEQUENCE = 260
DATA = Path(__file__).parent / "data"
# ODF export of Wikipedia article Hitchhiker's Guide to the Galaxy (CC-By-SA)
# Remark: the document is badly made: the pictures are not displayed in the
# text, but are sill inside the document !
SOURCE = "collection.odt"
OUTPUT_DIR = Path(__file__).parent / "recipes_output" / "found_pics"


def read_source_document() -> Document:
    """Return the source Document."""
    try:
        source = sys.argv[1]
    except IndexError:
        source = DATA / SOURCE
    return Document(source)


def read_pictures(document: Document) -> list[Path]:
    """Return the list of files retrieved from the document."""
    parts = document.parts
    print("ODF parts of the document:")
    print(pformat(parts))
    print()

    # we use the get_part function from odfdo to get the actual content
    # of the image, to copy the images out of the .odt file:
    OUTPUT_DIR.mkdir(parents=True, exist_ok=True)
    # images are DrawImage instances
    for draw_image in document.body.images:
        # where is the image actual content in the file:
        url = draw_image.url
        image_content = document.get_part(url)
        origin_path = Path(url)
        destination_path = OUTPUT_DIR / origin_path.name
        destination_path.write_bytes(image_content)

    result = sorted(OUTPUT_DIR.glob("*"))
    print(f"Picture files in {OUTPUT_DIR}:")
    for file in result:
        print(file.name)
    return result


def main() -> None:
    document = read_source_document()
    path_list = read_pictures(document)
    test_unit(path_list)


def test_unit(path_list: list[Path]) -> None:
    # only for test suite:
    if "ODFDO_TESTING" not in os.environ:
        return

    assert len(path_list) == 9


if __name__ == "__main__":
    main()



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2013 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Hervé Cauwelier <herve@itaapy.com>
#          Romain Gauthier <romain@itaapy.com>
#          Jerome Dumonteil <jerome.dumonteil@itaapy.com>
"""TrackedChanges class for "text:tracked-changes" and related classes
(ChangeInfo, TextInsertion, TextChange...).
"""

from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, Any, Union, cast

from .element import FIRST_CHILD, LAST_CHILD, Element, register_element_class
from .elements_between import elements_between
from .mixin_dc_creator import DcCreatorMixin
from .mixin_dc_date import DcDateMixin
from .mixin_list import ListMixin
from .mixin_md import MDZap
from .mixin_toc import TocMixin
from .paragraph import Paragraph
from .section import SectionMixin

if TYPE_CHECKING:
    from .body import Body, Text


class TrackedChangesMixin(Element):
    """Mixin class for classes containing TrackedChanges.

    Used by the following classes:  "office:text", "style:footer",
    "style:footer-first", "style:footer-left", "style:header",
    "style:header-first" and "style:header-left".
    """

    def get_tracked_changes(self) -> TrackedChanges | None:
        """Return the tracked-changes part in the text body.

        Returns:
            TrackedChanges or None: The tracked changes element, or None if
                not found.
        """
        return cast(
            Union[None, TrackedChanges], self.get_element("//text:tracked-changes")
        )

    @property
    def tracked_changes(self) -> TrackedChanges | None:
        """The 'text:tracked-changes' element in the body.

        This property provides access to the main container for tracked
        changes within the element's scope.
        """
        return self.get_tracked_changes()


class ChangeInfo(Element, DcCreatorMixin, DcDateMixin):
    """Information about a change, "office:change-info".

    This element stores metadata about a change, including the author,
    timestamp, and any associated comments.

    Comments are stored as one or more "text:p" (Paragraph) elements and can
    be accessed as Paragraph objects or as plain text.

    Attributes:
        creator (str): The name of the author who made the change.
        date (datetime): The date and time when the change was made.
    """

    _tag = "office:change-info"

    def __init__(
        self,
        creator: str | None = None,
        date: datetime | None = None,
        **kwargs: Any,
    ) -> None:
        """Initializes the ChangeInfo element.

        Args:
            creator: The name of the author of the change. Defaults to "Unknown".
            date: The date and time of the change. Defaults to the current time if not provided.
        """
        super().__init__(**kwargs)
        if self._do_init:
            self.creator = creator or "Unknown"
            self.date = date

    def get_comments(self, joined: bool = True) -> str | list[str]:
        """Gets the text content of the comments.

        Args:
            joined: If True (the default), concatenates the text of
                all comment paragraphs into a single string. If False,
                returns a list of strings, one for each paragraph.

        Returns:
            str | list[str]: The comment text as a single string or a list
                of strings.
        """
        content = self.paragraphs
        text = [para.get_formatted_text(simple=True) for para in content]
        if joined:
            return "\n".join(text)
        return text

    def set_comments(self, text: str = "", replace: bool = True) -> None:
        """Sets the text content of the comments.

        Args:
            text: The new text for the comments.
            replace: If True (the default), the new text replaces any
                existing comments. If False, it is appended as a new
                paragraph.
        """
        if replace:
            for para in self.paragraphs:
                self.delete(para)
        para = Paragraph()
        para.append_plain_text(text)
        self.insert(para, xmlposition=LAST_CHILD)


class TextInsertion(Element):
    """Represents a text insertion, "text:insertion".

    This element contains metadata about an insertion, including the author,
    date, and optional comments. It is linked to the actual inserted content
    in the document body via its parent "text:changed-region".
    """

    _tag = "text:insertion"

    def get_deleted(
        self,
        as_text: bool = False,
        no_header: bool = False,
    ) -> str | list[Element] | None:
        """Returns None, as this is a text insertion.

        This method is for consistency with other change types.

        Args:
            as_text: Ignored.
            no_header: Ignored.

        Returns:
            None | str: Always None for list return, empty string for text.
        """
        if as_text:
            return ""
        return None

    def get_inserted(
        self,
        as_text: bool = False,
        no_header: bool = False,
        clean: bool = True,
    ) -> str | Element | list[Element] | None:
        """Gets the content that was inserted.

        This is a shortcut to find the corresponding 'text:change-start' and
        'text:change-end' tags and return the content between them.

        Args:
            as_text: If True, returns the content as a plain text string.
            no_header: If True, converts any 'text:h' (heading)
                elements to 'text:p' (paragraph) elements.
            clean: If True, filters out unwanted elements from the result.

        Returns:
            str | Element | list[Element] | None: The inserted content,
                which can be a single element, a list of elements, a string,
                or None if not found.
        """
        current = self.parent  # text:changed-region
        if not isinstance(current, TextChangedRegion):
            raise TypeError("Missing parent TextChangedRegion")
        idx = current.get_id()
        body: Body | Element = self.document_body or self.root
        text_change = body.get_text_change_start(idx=idx)
        if not text_change:
            raise ValueError  # pragma: nocover
        return text_change.get_inserted(
            as_text=as_text, no_header=no_header, clean=clean
        )

    def get_change_info(self) -> ChangeInfo | None:
        """Gets the 'office:change-info' child of this element.

        Returns:
            ChangeInfo | None: The ChangeInfo element, or None if not found.
        """
        return cast(
            Union[None, ChangeInfo], self.get_element("descendant::office:change-info")
        )

    def set_change_info(
        self,
        change_info: Element | None = None,
        creator: str | None = None,
        date: datetime | None = None,
        comments: Element | list[Element] | None = None,
    ) -> None:
        """Sets the 'office:change-info' for this insertion.

        If `change_info` is not provided, a new one is created using the
        other arguments. Any existing change info is replaced.

        Args:
            change_info: An existing ChangeInfo element
                to set.
            creator: The name of the author. Defaults to
                'Unknown'.
            date: The date and time of the change.
                Defaults to the current time.
            comments: A Paragraph or list of Paragraphs to add as comments.
        """
        if change_info is None:
            new_change_info = ChangeInfo(creator, date)
            if comments is not None:
                if isinstance(comments, Element):
                    # single paragraph comment
                    comments_list = [comments]
                else:
                    comments_list = comments
                # assume iterable of Paragraph
                for paragraph in comments_list:
                    if not isinstance(paragraph, Paragraph):
                        raise TypeError(f"Not a Paragraph: '{paragraph!r}'")
                    new_change_info.insert(paragraph, xmlposition=LAST_CHILD)
        else:
            if not isinstance(change_info, ChangeInfo):
                raise TypeError(f"Not a ChangeInfo: '{change_info!r}'")
            new_change_info = change_info

        old = self.get_change_info()
        if old is not None:
            self.replace_element(old, new_change_info)
        else:
            self.insert(new_change_info, xmlposition=FIRST_CHILD)


class TextDeletion(ListMixin, TocMixin, SectionMixin, TextInsertion):
    """Represents a text deletion, "text:deletion".

    This element contains metadata about a deletion (author, date, comments)
    and stores the actual content that was deleted. The position in the
    document where the deletion occurred is marked by a "text:change"
    element.
    """

    _tag = "text:deletion"

    def get_deleted(
        self,
        as_text: bool = False,
        no_header: bool = False,
    ) -> str | list[Element] | None:
        """Gets the content that was deleted.

        Args:
            as_text: If True, returns the content as a plain text
                string.
            no_header: If True, converts any 'text:h' (heading)
                elements to 'text:p' (paragraph) elements.

        Returns:
            str | list[Element] | None: The deleted content, which is
                typically a list of Paragraph or Header elements, a string,
                or None.
        """
        children = self.children
        inner = [elem for elem in children if elem.tag != "office:change-info"]
        if no_header:  # crude replace t:h by t:p
            print("noheaders")
            new_inner = []
            for element in inner:
                if element.tag == "text:h":
                    children = element.children
                    text = element.text
                    para = Element.from_tag("text:p")
                    para.text = text
                    for child in children:
                        para.append(child)  # pragma: nocover
                    new_inner.append(para)
                else:
                    new_inner.append(element)
            inner = new_inner
        if as_text:
            return "\n".join([elem.get_formatted_text(context=None) for elem in inner])
        return inner

    def set_deleted(self, paragraph_or_list: Element | list[Element]) -> None:
        """Sets the content that was deleted.

        This method replaces any existing deleted content within this element.

        Args:
            paragraph_or_list: A Paragraph,
                Header, or a list of such elements representing the deleted
                content.
        """
        for element in self.get_deleted():  # type: ignore
            self.delete(element)  # type: ignore
        if isinstance(paragraph_or_list, Element):
            paragraph_or_list = [paragraph_or_list]
        for element in paragraph_or_list:
            self.append(element)

    def get_inserted(
        self,
        as_text: bool = False,
        no_header: bool = False,
        clean: bool = True,
    ) -> str | Element | list[Element] | None:
        """Returns None, as this is a text deletion.

        This method is for consistency with other change types.

        Args:
            as_text: Ignored.
            no_header: Ignored.
            clean: Ignored.

        Returns:
            None | str: Always None for list return, empty string for text.
        """
        if as_text:
            return ""
        return None


class TextFormatChange(TextInsertion):
    """Represents a change in text formatting, "text:format-change".

    This element marks a change in formatting attributes. The actual region
    of the change is defined by "text:change-start" and "text:change-end"
    elements.

    Note: This element itself does not contain the specific formatting
    changes, only metadata about the change event.
    """

    _tag = "text:format-change"


class TextChangedRegion(Element):
    """A container for a single tracked change, "text:changed-region".

    This element links a change marker in the document body (like
    "text:change-start") to the details of the change (like
    "text:insertion" or "text:deletion"). It contains exactly one of:
    TextInsertion, TextDeletion, or TextFormatChange.

    The 'xml:id' or 'text:id' of this element is referenced by the
    corresponding change marker elements.

    Warning:
        This implementation expects that a 'text:changed-region' is
        referenced only once, which differs from the ODF 1.2 specification.
    """

    _tag = "text:changed-region"

    def get_change_info(self) -> ChangeInfo | None:
        """Gets the ChangeInfo from the underlying change element.

        This is a shortcut to access the 'office:change-info' of the
        child (e.g., TextInsertion).

        Returns:
            ChangeInfo | None: The ChangeInfo element, or None if not found.
        """
        return cast(
            Union[None, ChangeInfo], self.get_element("descendant::office:change-info")
        )

    def set_change_info(
        self,
        change_info: Element | None = None,
        creator: str | None = None,
        date: datetime | None = None,
        comments: Element | list[Element] | None = None,
    ) -> None:
        """Sets the ChangeInfo on the underlying change element.

        This is a shortcut to set the 'office:change-info' of the child
        (e.g., TextInsertion). See `TextInsertion.set_change_info()` for
        more details.

        Args:
            change_info: An existing ChangeInfo element.
            creator: The author's name.
            date: The date of the change.
            comments: Comments to add.
        """
        child = self.get_change_element()
        if not child:
            raise ValueError("Empty TextChangedRegion")
        child.set_change_info(  # type: ignore
            change_info=change_info, creator=creator, date=date, comments=comments
        )

    def get_change_element(self) -> Element | None:
        """Gets the underlying change element.

        This will be one of TextInsertion, TextDeletion, or TextFormatChange.

        Returns:
            Element | None: The change element, or None if not found.
        """
        request = (
            "descendant::text:insertion "
            "| descendant::text:deletion"
            "| descendant::text:format-change"
        )
        return self._filtered_element(request, 0)

    def _get_text_id(self) -> str | None:
        return self.get_attribute_string("text:id")

    def _set_text_id(self, text_id: str) -> None:
        self.set_attribute("text:id", text_id)

    def _get_xml_id(self) -> str | None:
        return self.get_attribute_string("xml:id")

    def _set_xml_id(self, xml_id: str) -> None:
        self.set_attribute("xml:id", xml_id)

    def get_id(self) -> str | None:
        """Gets the "text:id" attribute of the region.

        Returns:
            str | None: The ID of the changed region.
        """
        return self._get_text_id()

    def set_id(self, idx: str) -> None:
        """Sets both the "text:id" and "xml:id" attributes to the same value.

        Args:
            idx: The ID to set.
        """
        self._set_text_id(idx)
        self._set_xml_id(idx)


class TrackedChanges(MDZap, Element):
    """The main container for all tracked changes, "text:tracked-changes".

    This element holds all the 'text:changed-region' elements that describe
    the individual changes (insertions, deletions, format changes) within
    its scope (e.g., the document body or a header).

    If this element is absent, change tracking is considered disabled for that
    scope, and any change markers should be ignored.
    """

    _tag = "text:tracked-changes"

    def get_changed_regions(
        self,
        creator: str | None = None,
        date: datetime | None = None,
        content: str | None = None,
        role: str | None = None,
    ) -> list[Element]:
        """Gets a list of 'text:changed-region' elements matching criteria.

        Args:
            creator: Filter by the author's name.
            date: Filter by the date of the change.
            content: Filter by a regex match in the content.
            role: Filter by the type of change ('insertion', 'deletion', 'format-change').

        Returns:
            list[Element]: A list of matching TextChangedRegion elements.
        """
        changed_regions = self._filtered_elements(
            "text:changed-region",
            dc_creator=creator,
            dc_date=date,
            content=content,
        )
        if role is None:
            return changed_regions
        result: list[Element] = []
        for region in changed_regions:
            changed = region.get_change_element()  # type: ignore
            if not changed:
                continue  # pragma: nocover
            if changed.tag.endswith(role):
                result.append(region)
        return result

    def get_changed_region(
        self,
        position: int = 0,
        text_id: str | None = None,
        creator: str | None = None,
        date: datetime | None = None,
        content: str | None = None,
    ) -> Element | None:
        """Gets a single 'text:changed-region' element by position or criteria.

        Args:
            position: The index of the region to retrieve.
            text_id: Get the region with this specific ID.
            creator: Filter by the author's name.
            date: Filter by the date of the change.
            content: Filter by a regex match in the content.

        Returns:
            Element | None: The matching TextChangedRegion element, or None
                if not found.
        """
        return self._filtered_element(
            "text:changed-region",
            position,
            text_id=text_id,
            dc_creator=creator,
            dc_date=date,
            content=content,
        )


class TextChange(Element):
    """A text change position, "text:change".

    The TextChange "text:change" element marks a position in an empty region
    where text has been deleted.
    """

    _tag = "text:change"

    def get_id(self) -> str | None:
        """Gets the ID of the change this marker refers to.

        Returns:
            str | None: The 'text:change-id' attribute value.
        """
        return self.get_attribute_string("text:change-id")

    def set_id(self, idx: str) -> None:
        """Sets the ID of the change this marker refers to.

        Args:
            idx: The ID to set as 'text:change-id'.
        """
        self.set_attribute("text:change-id", idx)

    def _get_tracked_changes(self) -> TrackedChanges | None:
        body: Text | None = self.document_body  # type: ignore[assignment]
        if body and body.tag == "office:text":
            return body.get_tracked_changes()
        raise ValueError

    def get_changed_region(
        self,
        tracked_changes: TrackedChanges | None = None,
    ) -> Element | None:
        """Finds the 'text:changed-region' this marker is associated with.

        Args:
            tracked_changes: The parent tracked
                changes container to search in. If not provided, it is
                inferred from the document.

        Returns:
            Element | None: The associated TextChangedRegion element, or
                None if not found.
        """
        if not tracked_changes:
            tracked_changes = self._get_tracked_changes()
        idx = self.get_id()
        return tracked_changes.get_changed_region(text_id=idx)  # type: ignore

    def get_change_info(
        self,
        tracked_changes: TrackedChanges | None = None,
    ) -> Element | None:
        """Gets the 'office:change-info' for the change this marker refers to.

        Args:
            tracked_changes: The parent tracked changes container to search in.

        Returns:
            Element | None: The ChangeInfo element, or None if not found.
        """
        changed_region = self.get_changed_region(tracked_changes=tracked_changes)
        if not changed_region:
            return None  # pragma: nocover
        return changed_region.get_change_info()  # type: ignore

    def get_change_element(
        self,
        tracked_changes: TrackedChanges | None = None,
    ) -> Element | None:
        """Gets the underlying change element for this marker.

        This will be one of TextInsertion, TextDeletion, or TextFormatChange.

        Args:
            tracked_changes: The parent tracked changes container to search in.

        Returns:
            Element | None: The change element, or None if not found.
        """
        changed_region = self.get_changed_region(tracked_changes=tracked_changes)
        if not changed_region:
            return None  # pragma: nocover
        return changed_region.get_change_element()  # type: ignore

    def get_deleted(
        self,
        tracked_changes: TrackedChanges | None = None,
        as_text: bool = False,
        no_header: bool = False,
        clean: bool = True,
    ) -> Element | None:
        """Gets the deleted content associated with this change marker.

        This is a shortcut to find the related TextDeletion element and
        retrieve its content.

        Args:
            tracked_changes: The parent tracked changes container.
            as_text: Return content as a plain text string.
            no_header: Convert headings to paragraphs.
            clean: Ignored.

        Returns:
            Element | None: The deleted content.
        """
        changed = self.get_change_element(tracked_changes=tracked_changes)
        if not changed:
            return None  # pragma: nocover
        return changed.get_deleted(  # type: ignore
            as_text=as_text,
            no_header=no_header,
        )

    def get_inserted(
        self,
        as_text: bool = False,
        no_header: bool = False,
        clean: bool = True,
    ) -> str | Element | list[Element] | None:
        """Returns None, as 'text:change' marks a deletion point.

        Args:
            as_text: unused.
            no_header: unused.
            clean: unused.

        Returns:
            str | Element | list[Element] | None: Always returns None.
        """
        return None

    def get_start(self) -> TextChangeStart | None:
        """Returns None, as this is a single-point marker.

        Returns:
            TextChangeStart | None: Always returns None.
        """
        return None

    def get_end(self) -> TextChangeEnd | None:
        """Returns None, as this is a single-point marker.

        Returns:
            TextChangeEnd | None: Always returns None.
        """
        return None


class TextChangeEnd(TextChange):
    """End of a changed region, "text:change-end".

    The TextChangeEnd "text:change-end" element marks the end of a region with
    content where text has been inserted or the format has been changed.
    """

    _tag = "text:change-end"

    def get_start(self) -> TextChangeStart | None:
        """Gets the corresponding 'text:change-start' tag.

        Returns:
            TextChangeStart | None: The start marker with the same ID, or
                None if not found.
        """
        idx = self.get_id()
        parent = self.parent
        if parent is None:
            raise ValueError(
                "Can not find end tag: no parent available."
            )  # pragma: nocover
        body: Body | Element = self.document_body or self.root
        return body.get_text_change_start(idx=idx)

    def get_end(self) -> TextChangeEnd | None:
        """Returns self, as this is the end marker.

        Returns:
            TextChangeEnd: This element.
        """
        return self

    def get_deleted(self, *args: Any, **kwargs: Any) -> Element | None:
        """Returns None, as this marks an insertion or format change.

        Returns:
            None: Always returns None.
        """
        return None

    def get_inserted(
        self,
        as_text: bool = False,
        no_header: bool = False,
        clean: bool = True,
    ) -> str | Element | list[Element] | None:
        """Gets the content between the start and end change markers.

        Args:
            as_text: If True, returns the content as a plain text string.
            no_header: If True, converts any 'text:h' (heading) elements to 'text:p' (paragraph) elements.
            clean: If True, filters out unwanted elements from the result.

        Returns:
            str | Element | list[Element] | None: The inserted content.
        """

        # idx = self.get_id()
        start = self.get_start()
        end = self.get_end()
        if end is None or start is None:
            if as_text:
                return ""
            return None
        body: Body | Element = self.document_body or self.root
        return elements_between(
            body, start, end, as_text=as_text, no_header=no_header, clean=clean
        )


class TextChangeStart(TextChangeEnd):
    """Start of a changed region, "text:change-start".

    The TextChangeStart "text:change-start" element marks the start of a region
    with content where text has been inserted or the format has been changed.
    """

    _tag = "text:change-start"

    def get_start(self) -> TextChangeStart:
        """Returns self, as this is the start marker.

        Returns:
            TextChangeStart: This element.
        """
        return self

    def get_end(self) -> TextChangeEnd:
        """Gets the corresponding 'text:change-end' tag.

        Returns:
            TextChangeEnd | None: The end marker with the same ID, or
                None if not found.
        """
        idx = self.get_id()
        parent = self.parent
        if parent is None:
            raise ValueError(
                "Can not find end tag: no parent available."
            )  # pragma: nocover
        body: Body | Element = self.document_body or self.root
        return body.get_text_change_end(idx=idx)  # type: ignore

    def delete(
        self,
        child: Element | None = None,
        keep_tail: bool = True,
    ) -> None:
        """Deletes the element from the XML tree.

        For a TextChangeStart, this also deletes the corresponding
        'text:change-end' tag if it exists.

        Args:
            child: A specific child to delete. If None,
                the element itself is deleted.
            keep_tail: If True, the text that follows the element
                is preserved. Defaults to True.
        """
        if child is not None:  # act like normal delete
            return super().delete(child, keep_tail)  # pragma: nocover
        idx = self.get_id()
        if self.parent is None:
            raise ValueError("cannot delete the root element")  # pragma: nocover
        body: Body | Element = self.document_body or self.root
        end = body.get_text_change_end(idx=idx)
        if end:  # pragma: nocover
            end.delete()
        # act like normal delete
        super().delete()


register_element_class(ChangeInfo)
register_element_class(TextInsertion)
register_element_class(TextDeletion)
register_element_class(TextFormatChange)
register_element_class(TextChangedRegion)
register_element_class(TrackedChanges)
register_element_class(TextChange)
register_element_class(TextChangeEnd)
register_element_class(TextChangeStart)



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Hervé Cauwelier <herve@itaapy.com>

import warnings
from collections.abc import Iterable

import pytest

from odfdo.const import ODF_CONTENT
from odfdo.container import Container
from odfdo.style import Style
from odfdo.xmlpart import XmlPart


@pytest.fixture
def style_element(samples) -> Iterable[Style]:
    container = Container(samples("example.odt"))
    content_part = XmlPart(ODF_CONTENT, container)
    query = '//style:style[@style:family="paragraph"][1]'
    yield content_part.get_element(query)


def test_odf_style(style_element):
    assert isinstance(style_element, Style)


def test_get_style_properties(style_element):
    style = style_element
    properties = style.get_properties()
    assert isinstance(properties, dict)
    assert len(properties) == 12
    assert properties["fo:margin-left"] == "0cm"


def test_get_style_properties_area(style_element):
    style = style_element
    properties = style.get_properties(area="text")
    assert isinstance(properties, dict)
    assert len(properties) == 1
    assert properties["fo:hyphenate"] == "false"


def test_get_style_properties_bad_element(samples):
    container = Container(samples("example.odt"))
    content_part = XmlPart(ODF_CONTENT, container)
    paragraph_element = content_part.get_element("//text:p[1]")
    with pytest.raises(AttributeError):
        paragraph_element.get_properties()


def test_get_style_properties_bad_area(style_element):
    style = style_element
    assert style.get_properties(area="toto") is None


def test_set_style_properties(style_element):
    style = style_element
    # will not be allowed in paragraph-property (it's a text-property)
    with warnings.catch_warnings():
        warnings.simplefilter("ignore", category=UserWarning)
        style.set_properties({"fo:color": "#f00"})
    properties = style.get_properties()
    assert len(properties) == 12  # unchanged
    assert properties.get("fo:color") is None


def test_set_style_properties_area(style_element):
    style = style_element
    style.set_properties({"fo:color": "#f00"}, area="text")
    properties = style.get_properties(area="text")
    assert len(properties) == 2
    assert properties["fo:color"] == "#f00"


def test_set_style_properties_new_area(style_element):
    style = style_element
    properties = style.get_properties(area="chart")
    assert properties is None
    style.set_properties({"fo:color": "#f00"}, area="chart")
    properties = style.get_properties(area="chart")
    assert len(properties) == 1
    assert properties["fo:color"] == "#f00"


def test_del_style_properties(style_element):
    style = style_element
    style.del_properties(["fo:margin-left"])
    properties = style.get_properties()
    assert len(properties) == 11
    with pytest.raises(KeyError):
        _dummy = properties["fo:margin-left"]


def test_del_style_properties_bad_area(style_element):
    style = style_element
    with pytest.raises(ValueError):
        style.del_properties(area="foobar")



#!/usr/bin/env python
"""Minimal example of how to add an item to a list."""

import os

from odfdo import List, ListItem

_DOC_SEQUENCE = 28


def generate_list() -> List:
    """Return a small List."""
    drink_list = List(["chocolate", "coffee"])
    item = ListItem("tea")
    drink_list.append(item)

    return drink_list


def main() -> None:
    some_list = generate_list()
    test_unit(some_list)


def test_unit(some_list: List) -> None:
    # only for test suite:
    if "ODFDO_TESTING" not in os.environ:
        return

    assert str(some_list).strip() == "-  chocolate\n -  coffee\n -  tea"


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""Minimal example of setting a page footer using StyleMasterPage.set_page_footer().

Note: the created footer uses the current footer style, to change that
footer style, use the method  set_footer_style() on the 'page-layout'
style family.
"""

import os
from pathlib import Path

from odfdo import Document, Header, Paragraph, Tab, VarPageNumber

_DOC_SEQUENCE = 62
OUTPUT_DIR = Path(__file__).parent / "recipes_output" / "styled4"
TARGET = "document.odt"


def save_new(document: Document, name: str) -> None:
    """Save a recipe result Document."""
    OUTPUT_DIR.mkdir(parents=True, exist_ok=True)
    new_path = OUTPUT_DIR / name
    print("Saving:", new_path)
    document.save(new_path, pretty=True)


def make_document() -> Document:
    """Generate a short document with a page footer."""
    text_1 = (
        "Lorem ipsum dolor sit amet,\n\t"
        "consectetuer adipiscing elit.\n\tSed"
        "non risus.\n\tSuspendisse lectus tortor,\n"
        "ndignissim sit amet, \nadipiscing nec,"
        "\nultricies sed, dolor.\n\n"
        " Cras elementum ultrices diam. Maecenas ligula massa,"
        "varius a,semper congue, euismod non,"
        " mi. Proin porttitor, orci nec nonummy"
        "molestie, enim est eleifend mi,"
        " non fermentum diam nisl sit amet erat."
    )

    document = Document("text")
    body = document.body
    body.clear()
    body.append(Header(1, "Some Title"))
    body.append(Paragraph(text_1))

    # looking for the current "master-page" style, it is probably
    # named "Standard". If not found, search with something like:
    # print([s for s in document.get_styles() if s.family == "master-page"])
    page_style = document.get_style("master-page", "Standard")

    # Another method is (if "Standard" is not the right name):
    # master page are defined in Styles xml part:
    styles = document.styles
    master_pages = styles.master_pages
    # maybe thre are several master pages (ie.: first pages, and remaining
    # of document)
    # let assume we wand the frist one.
    page_style = master_pages[0]

    # The footer can be a Paragraph or a list of Paragraphs:
    first_line = Paragraph("\tA first footer line")
    second_line = Paragraph("Second line")
    second_line.append(Tab())
    second_line.append(Tab())
    second_line.append(VarPageNumber())
    second_line.append(".")
    my_footer = [first_line, second_line]

    page_style.set_page_footer(my_footer)

    # important: insert again the modified style
    document.insert_style(page_style)

    return document


def main() -> None:
    document = make_document()
    test_unit(document)
    save_new(document, TARGET)


def test_unit(document: Document) -> None:
    # only for test suite:
    if "ODFDO_TESTING" not in os.environ:
        return
    from odfdo import StyleMasterPage

    assert len([s for s in document.get_styles() if s.family == "master-page"]) >= 1
    page_style = document.get_style("master-page", "Standard")
    assert isinstance(page_style, StyleMasterPage)
    footer = page_style.get_page_footer()
    content = footer.serialize()
    assert "A first footer" in content
    assert "Second line" in content


if __name__ == "__main__":
    main()



# Copyright 2018-2026 Jérôme Dumonteil
# Authors (odfdo project): jerome.dumonteil@gmail.com

from odfdo.version import __version__


def test_version():
    assert __version__
    assert isinstance(__version__, str)



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
from odfdo.style import Style
from odfdo.style_defaults import (
    default_boolean_style,
    default_currency_style,
    default_date_style,
    default_number_style,
    default_percentage_style,
    default_time_style,
)


def test_default_boolean_style_create():
    style = default_boolean_style()
    assert isinstance(style, Style)


def test_default_currency_style_create():
    style = default_currency_style()
    assert isinstance(style, Style)


def test_default_date_style_create():
    style = default_date_style()
    assert isinstance(style, Style)


def test_default_number_style_create():
    style = default_number_style()
    assert isinstance(style, Style)


def test_default_percentage_style_create():
    style = default_percentage_style()
    assert isinstance(style, Style)


def test_default_time_style_create():
    style = default_time_style()
    assert isinstance(style, Style)


def test_default_boolean_style_name():
    style = default_boolean_style()
    assert style.name == "lpod-default-boolean-style"


def test_default_currency_style_name():
    style = default_currency_style()
    assert style.name == "lpod-default-currency-style"


def test_default_date_style_name():
    style = default_date_style()
    assert style.name == "lpod-default-date-style"


def test_default_number_style_name():
    style = default_number_style()
    assert style.name == "lpod-default-number-style"


def test_default_percentage_style_name():
    style = default_percentage_style()
    assert style.name == "lpod-default-percentage-style"


def test_default_time_style_name():
    style = default_time_style()
    assert style.name == "lpod-default-time-style"


def test_default_boolean_style_tag():
    style = default_boolean_style()
    assert style.tag == "number:boolean-style"


def test_default_currency_style_tag():
    style = default_currency_style()
    assert style.tag == "number:currency-style"


def test_default_date_style_tag():
    style = default_date_style()
    assert style.tag == "number:date-style"


def test_default_number_style_tag():
    style = default_number_style()
    assert style.tag == "number:number-style"


def test_default_percentage_style_tag():
    style = default_percentage_style()
    assert style.tag == "number:percentage-style"


def test_default_time_style_tag():
    style = default_time_style()
    assert style.tag == "number:time-style"
